/**
 *@file ISTrim.cc
 *
 *   Stands for Independent Set Trim. Uses the Jaccard coefficient between community memberships
 *     to prune community structure. If two communities have a Jaccard coefficient above a certain
 *     (user-given) threshold, only one community is kept.
 *
 *@input  Text file consisting of community memberships (one community per line, members delimited by some character)
 *@output   A new text file, in the same format, with the communities that were too similar pruned out.
 *
 *This program is free software: you can redistribute it and/or modify                                                                 
 *    it under the terms of the GNU General Public License as published by                                                             
 *    the Free Software Foundation, either version 3 of the License, or                                                                
 *    (at your option) any later version.                                                                                              
 *                                                                                                                                     
 *    This program is distributed in the hope that it will be useful,                                                                  
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of                                                                   
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                                    
 *    GNU General Public License for more details.                                                                                     
 *                                                                                                                                     
 *    You should have received a copy of the GNU General Public License                                                                
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.                                                            
 *                                                                                                                                     
 *                                                                                                                                     
 *@author James Thompson                                                                                                               
 *Copyright 2014 James Thompson 
 *
 * TODO:
 *     Add option to merge communities if similarity is above some threshold
 *
 */

#include "Toolbox/Util/Parameters.h"
#include "Toolbox/Util/StringEx.h"
#include "Toolbox/Util/IOX.h"
#include "Toolbox/Util/STX.h"

#include <algorithm>

#include <tr1/memory>

/**
 *@struct cnode
 *
 *  When calculating the independent set to use, nodes will have to be 
 *    moved around (sorted). This structure is used to keep the node ID and degree
 *    together for retrieval after sorting.
 */
struct cnode{
  /**
   * @fn cnode (int d, int i)
   *
   * Constructor - Each node needs a degree and an ID or else it's not being used correctly
   */
  cnode (int d, int i):degree(d), ID(i){};

  //Member variables
  int degree;
  int ID;
};

/**
 *@fn bool compcnode(const cnode* A, const cnode* B)
 *
 *  Helper function to use when sorting vectors or sets of 'struct cnode's
 */
bool compcnode(const cnode* A, const cnode* B){
  return A->degree < B->degree;
}

/**
 * @fn int main ( int argc, char** argv )
 *
 * Program Entry Point
 *
 * @return Nothing special
 */
int main ( int argc, char** argv ){
  string filename, output, delims;
  bool help;
  double JaccardThresh;

  //Read in parameters
  std::tr1::shared_ptr < Parameters > P (new Parameters());
  P->Read(argc, argv);
  
  P->set<string> (&filename, "i", "communities.dat");
  P->set<string> (&output, "o", "results.dat");
  P->set<string> (&delims, "dl", " \t");
  P->set<double> (&JaccardThresh, "thr", 0.8);
  P->boolset(&help, "h");
  
  if (help || (argc == 1)){
    cerr << "./ISTrim\n\t-i Input File (communities.dat)\n\t-o Output File (results.dat)\n\t-dl Delimiters to use ([tab] and [space])\n\t-thr Threshold for pruning (0.8)\n\t" << endl;
    exit(2);
  }

  //Read in communities
  ifstream fin;
  openFileHarsh(&fin, filename);
  vector < string > fields;

  vector < set < string > > communities;

  while ( fline_tr ( &fin, &fields, delims ) ){
    set < string > current (fields.begin(), fields.end());
    communities.push_back(current);
  }

  //Construct network of communities
  map < int, set < int > > network;
  vector < cnode* > degrees;
  for (unsigned int i = 0; i < communities.size(); i++){
    degrees.push_back(new cnode(0, i));
  }
  
  for (unsigned int i = 0; i < communities.size(); i++){
    for (unsigned int j = i + 1; j < communities.size(); j++){
      if (Jaccard < string >(communities[i], communities[j]) > JaccardThresh){
	Insert <int, int> (network, i, j);
	Insert <int, int> (network, j, i);
	
	++degrees[i]->degree;
	++degrees[j]->degree;
      }
    }
  }

  sort (degrees.begin(), degrees.end(), compcnode);                                

  //Find Independant Set (Greedy)
  set < int > forbidden, chosen;
  set < int >::iterator it_f;
  map < int, set < int > >::iterator it_n;  

  for (unsigned int i = 0; i < degrees.size(); i++){                               //Go through communities, low to high degrees
    if ( forbidden.find(degrees[i]->ID) == forbidden.end() ){                      //Make sure not already chosen / removed
      chosen.insert(degrees[i]->ID);                                               //Greedily take the community
      forbidden.insert(degrees[i]->ID);                                            
      
      if ( (it_n = network.find(degrees[i]->ID)) != network.end() ){               //Remove the neighbors of this node
	for ( it_f = it_n->second.begin(); it_f != it_n->second.end(); it_f++){
	  forbidden.insert(*it_f);
	}
      }
    }
  }
  
  //Print results
  ofstream fout(output.c_str());
  set < string >::iterator it_s;

  for (it_f = chosen.begin(); it_f != chosen.end(); it_f++){
    it_s = communities[*it_f].begin();
    fout << *it_s;
    for (++it_s; it_s != communities[*it_f].end(); it_s++){
      fout << ", " << *it_s;
    }
    fout << endl;
  }

  fout.close();
}
