/**
 *@file OrderTrim.cc
 *
 *  Called Order Trim because, originally, the idea was to sort the communities and take the top 10% or
 *    or bottom 10% of communities based on some metric. As it turns out, this was changed to get
 *    all communities whose metric falls in a range, so there is no actual ordering going on here.
 *
 *@input  Community text file: One community per line, members separated by some delimiter.
 *@output Same format of text file, with communities whose metric fell outside the given range missing.
 *
 *
 *This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *@author James Thompson
 *Copyright 2014 James Thompson
 */
#include "Toolbox/Util/IOX.h"
#include "Toolbox/Util/Parameters.h"
#include "Toolbox/Util/StringEx.h"
#include "Toolbox/Util/STX.h"

#include "Structure/Density.h"
#include "Structure/Network.h"

#include <tr1/memory>

char DTYPE = 'c';

/**
 *@fn double c_size(std::tr1::shared_ptr < Community > C, std::tr1::shared_ptr < Parameters > P)
 *
 *  Retrieves the size of the given community.
 *
 * @param std::tr1::shared_ptr < Community > C Community of interest
 * @param std::tr1::shared_ptr < Parameters > P Extra parameters that might be necessary ( not needed for this particular function )
 */
double c_size(std::tr1::shared_ptr < Community > C, std::tr1::shared_ptr < Parameters > P){
  return C->size();  
}

/**
 *@fn double c_dens(std::tr1::shared_ptr < Community > C, std::tr1::shared_ptr < Parameters > P)
 *
 *  Retrieves the density of the given community.
 *
 * @param std::tr1::shared_ptr < Community > C Community of interest
 * @param std::tr1::shared_ptr < Parameters > P Extra parameters that might be necessary ( lambda value for density )
 */
double c_dens(std::tr1::shared_ptr < Community > C, std::tr1::shared_ptr < Parameters > P){
  std::tr1::shared_ptr < Density > D = Density::Create(DTYPE, P);

  return D->calc(C);
}

/**
 * Entry point for the program
 */
int main(int argc, char** argv){
  char feature;
  string filename, output, networkfile, ndl, cdl;

  double lower_bound, upper_bound;
  bool low, up, fullData, help;

  //Read in command line
  std::tr1::shared_ptr < Parameters > P (new Parameters());
  P->Read(argc, argv);
  
  P->set < string >(&filename, "i", "communities.dat");
  P->set < string >(&networkfile, "n", "network.dat");
  P->set < string >(&ndl, "ndl", " \t");
  P->set < string >(&cdl, "cdl", " \t");
  P->set < string >(&output, "o", "results.dat");
  P->set < char > (&feature, "f", 's');
  
  P->set < double > (&lower_bound, "lb", 0.5);
  P->set < double > (&upper_bound, "ub", 0);
  P->boolset (&low, "lb");
  P->boolset (&up, "ub");
  P->boolset (&fullData, "all");
  P->boolset (&help, "h");

  if (help || (argc == 1)){
    cerr << "./otrim (defaults in parentheses)\n\t-i Community file (communities.dat)\n\t-n Network file (network.dat)\n\t-o Output file (results.dat)\n\t-f";
    cerr << " Feature: s is size, d is (complex) density (include a -l lambda value)  (default:s)\n\t";
    cerr << "-lb Lower Bound (none)\n\t-ub Upper Bound (none)\n\t-all Show extra information in output (false)\n\t-h This help\n\t";
    cerr << "-ndl Network file delimiter ([tab] and [space])\n\t-cdl Community file delimiter ([tab] and [space])\n";
    exit(3);
  }

  // Set the evaluation function depending on user inputs
  double (*eval)(std::tr1::shared_ptr < Community >, std::tr1::shared_ptr < Parameters >);

  switch (feature){
  case 's':
    eval = &c_size;
    break;

  case 'd':
    eval = &c_dens;
    break;
  }

  std::tr1::shared_ptr < Graph > G (new UndirectedGraph());                    //Set up network
  G->LoadStructure(networkfile, ",", 0, false);
  G->NormalizeWeights();

  ifstream fin;                                                                //Analyze while reading in
  openFileHarsh(&fin, filename);

  ofstream fout(output.c_str());

  vector < string > fields;
  while (fline_tr(&fin, &fields, ",")){
    set < Graph::vertex, cmp_tr1_string > current;                             //Create right format
    for (uint i = 0; i < fields.size(); i++){
      Graph::vertex V(new string(fields[i]));
      current.insert(V);
    }
    
    std::tr1::shared_ptr<Community> tC(new Community(current, G));
    double val = eval(tC, P);                                                  //Calc value

    if ( (!(low && (val < lower_bound))) && (!(up && (val > upper_bound))) ){  //If the value is in range - keep community
      if ( fullData )
	fout << val << " : ";
      tC->print(fout);
    }
  }

  fout.close();
}
