/**
 *@file Community.cc
 *
 *  Implementation of community representation.
 *
 *  NOTE: Edges in directed networks that start from outside the community and point into it are not considered as 
 *    'external edges'(yet?)
 */

#include "Community.h"

/**
  * @fn Community::Community(const std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string > nodes)
  * @param c Set of vertices to initialize this community as
  */
Community::Community(const std::set< Graph::vertex, cmp_tr1_string > nodes, std::tr1::shared_ptr < Graph > Gr):members(nodes), G(Gr){
  calcWin();
  calcWout();
}

/**
  * @fn Community::Community(const Community& other)
  * @param other Community to copy
  * 
  * Copy Constructor
  */
Community::Community(const Community& other){
  members = other.members;
  calcWin();
  calcWout();
}

/**
  * @fn Community(const Community* other)
  * @param other Community to copy
  * 
  * Copy Constructor
  */
Community::Community(const Community* other){
  members = other->members;
  calcWin();
  calcWout();
}

/**
  * @fn bool Community::operator<(const Community& other)
  * @param other Community to compare to
  * 
  * Function to use in order to compare communities (for use in containers)
  * 
  * @return True if this community alphabetically preceeds the other
  * 		False otherwise
  */
bool Community::operator<(const Community& other)
{
  struct cmp_tr1_string cts;
  bool A, B;  
  
  std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::iterator it_m2;
  
  for (it_m = members.begin(), it_m2 = other.members.begin(); ((it_m != members.end()) && (it_m2 != other.members.end())); it_m++, it_m2++){
    if ((A = cts(*it_m, *it_m2)) != (B = cts(*it_m2, *it_m))){
      return A;
    }
  }
  
  if (it_m != members.end()){
    return true;
  } else if (it_m2 != other.members.end()){
    return false;
  } else {
    return false;
  }
}

/**
  * @fn std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator Community::first()
  * @return Iterator to the first member of this community
  */
std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string >::const_iterator Community::first()
{
  return members.begin();
}

std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string >::const_iterator Community::first() const
{
  return members.begin();
}

/**
  * @fn std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator Community::last()
  * @return Iterator to the last member of this community
  */
std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string >::const_iterator Community::last()
{
  return members.end();
}

std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string >::const_iterator Community::last() const
{
  return members.end();
}

/**
  * @fn bool Community::hasMember(std::tr1::shared_ptr<std::string> node)
  * @param node A vertex
  * 
  * @return True if the given node is in the community
  * 		False otherwise
  */
bool Community::hasMember(std::tr1::shared_ptr< std::string > node)
{
  if (members.find(node) != members.end())
    return true;
  
  return false;
}

/**
  * @fn bool Community::hasMember(std::tr1::shared_ptr<std::string> node)
  * @param node A vertex
  * 
  * @return True if the given node is in the community
  * 		False otherwise
  */
bool Community::hasMember(std::tr1::shared_ptr< const std::string > node)
{
  if (members.find(node) != members.end())
    return true;
  
  return false;
}

/**
  * @fn int Community::size()
  * @return Number of members in the community
  */
int Community::size()
{
  return members.size();
}

/**
  * @fn void Community::addMember(std::tr1::shared_ptr<std::string> node)
  * @param node A vertex
  * 
  * Inserts the given node into the community
  * 	There is no connectivity checking.
  *
  * @return true if the insertion was successful (a new node)
  */
bool Community::addMember(std::tr1::shared_ptr< const std::string > node)
{
  if (G->hasVertex(node)){
    pair < set < Graph::vertex, cmp_vertex >::iterator, bool > res = members.insert(node);
    if ( res.second ){
      pair < double, double > res = InOutDeg(node);
      w_in = w_in + res.first;
      w_out = w_out + res.second - res.first;
    }
    return res.second;
  }
  
  return false;
}

/**
  * @fn void Community::removeMember(std::tr1::shared_ptr<std::string> node)
  * @param node A vertex
  * 
  * Removes the given node from the community (if it was a member previously)
  * 	There is no connectivity / error checking done by this function at this time
  *
  * @return true if removal was successful (will always be successful)
  */
bool Community::removeMember(std::tr1::shared_ptr< const std::string > node)
{
  //Node already not a member
  if ((it_m = members.find(node)) == members.end())
    return true;
  
  members.erase(it_m);
 
  pair < double, double > res = InOutDeg( node );
  w_in = w_in - res.first;
  w_out = w_out - res.second + res.first;

  return true;
}

/**
  * @fn bool Community::cmp_comm::operator()(const std::tr1::shared_ptr<Community> A, const std::tr1::shared_ptr<Community> B)
  * Comparison structure to keep Communities alphabetically ordered in STL containers
  */
bool Community::cmp_comm::operator()(const std::tr1::shared_ptr<Community> A, const std::tr1::shared_ptr<Community> B)
{
  return ((*A) < (*B));
}

/**
  * @fn void Community::print()
  * 
  * Send member list to standard output on a single line (separated by commas)
  */
void Community::print()
{
  static_cast<const Community*>(this)->print();
}

/**
  * @fn void Community::print() const
  * 
  * Send member list to standard output on a single line (separated by commas)
  */
void Community::print() const
{
  if (members.size() == 0){
      std::cout << "{Empty}\n-------------------------------" << std::endl;
      return;
  }
  
  it_m = members.begin();
  std::cout << **it_m;
  for (++it_m; it_m != members.end(); it_m++){
    std::cout << ", " << **it_m;
  }
  
  std::cout << std::endl;
}

void Community::print(std::ostream& fout){
  static_cast<const Community*>(this)->print(fout);
}

void Community::print(std::ostream& fout) const{
  if (members.size() == 0){
    fout << "---------------------------------" << std::endl;
    return;
  }

  it_m = members.begin();
  fout << **it_m;
  for (++it_m; it_m != members.end(); it_m++){
    fout << ", " << **it_m;
  }

  fout << std::endl;
}

/**
   * @fn void Community::addChild(int len, double str, std::tr1::shared_ptr<Community> other)
   * @param len Length of the evolution chain (including the current community)
   * @param str Strength of tie with the next community in the chain
   * @param other Community to provide the next link in the chain
   * 
   * Add other community as a potential next evolution link, tracking the evolution length and the strength of the tie
   */
void Community::addChild(int len, double str, std::tr1::shared_ptr< Community > other)
{
  if ((it_chain = children.find(len)) != children.end()){
    if ((it_sch = it_chain->second.find(str)) != it_chain->second.end()){
      it_sch->second.insert(other);
    } else {
      std::set < std::tr1::shared_ptr< Community > > tmp;
      tmp.insert(other);
      it_chain->second.insert(std::pair< double, std::set< std::tr1::shared_ptr< Community > > >(str, tmp));
    }
  } else {
    std::set < std::tr1::shared_ptr< Community > > tmp;
    std::map < double, std::set < std::tr1::shared_ptr< Community > > > mtmp;
    tmp.insert(other);
    mtmp.insert(std::pair< double, std::set< std::tr1::shared_ptr< Community > > >(str, tmp));
    
    children.insert(std::pair<int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > >(len, mtmp));
  }
}

/**
   * @fn int Community::longest()
   * 
   * @return The length of the longest evolution starting from this community.
   */
int Community::longest()
{
  if (children.size() == 0)
    return 1;
  
  int result = 0;
  for (it_chain = children.begin(); it_chain != children.end(); it_chain++){
    if (it_chain->first > result)
      result = it_chain->first;
  }
  
  return result;
}

/**
   * @fn int Community::preLongest()
   * 
   * @return The length of the longest evolution ending at this community.
   */
int Community::preLongest()
{
  if (parents.size() == 0)
    return 1;
  
  int result = 0;
  for (it_chain = parents.begin(); it_chain != parents.end(); it_chain++){
    if (it_chain->first > result)
      result = it_chain->first;
  }
  
  return result;
}

/**
   * @fn void Community::addParent(int len, double str, std::tr1::shared_ptr<Community> other)
   * @param len Length of the evolution chain (including the current community)
   * @param str Strength of tie with the next community in the chain
   * @param other Community to provide the next link in the chain
   * 
   * Add other community as a potential parent evolution link, tracking the evolution length and the strength of the tie
   */
void Community::addParent(int len, double str, std::tr1::shared_ptr< Community > other)
{
  if ((it_chain = parents.find(len)) != parents.end()){
    if ((it_sch = it_chain->second.find(str)) != it_chain->second.end()){
      it_sch->second.insert(other);
    } else {
      std::set < std::tr1::shared_ptr< Community > > tmp;
      tmp.insert(other);
      it_chain->second.insert(std::pair< double, std::set< std::tr1::shared_ptr< Community > > >(str, tmp));
    }
  } else {
    std::set < std::tr1::shared_ptr< Community > > tmp;
    std::map < double, std::set < std::tr1::shared_ptr< Community > > > mtmp;
    tmp.insert(other);
    mtmp.insert(std::pair< double, std::set< std::tr1::shared_ptr< Community > > >(str, tmp));
    
    parents.insert(std::pair<int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > >(len, mtmp));
  }
}

/**
   * @fn void Community::printEvolution(bool careAboutParents, const int& minSize)
   * @param careAboutParents Flag to distinguish whether or not to print only evolutions that begin with this community or all
   * 	partial evolutions that pass through this community
   * @param minSize Minimum remaining length of an evolution required to print it out
   * 
   * Will print one community per line.
   * 
   */
bool Community::printEvolution(bool careAboutParents, const int& minSize)
{
  if (((parents.size() > 0) && careAboutParents))
    return false;
  if (children.size() > 0){
   std::map < int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > >::reverse_iterator it_rchain = children.rbegin();
   if (minSize > it_rchain->first)
     return false;
   print();
    std::map < double, std::set < std::tr1::shared_ptr< Community > > >::reverse_iterator it_rsch = it_rchain->second.rbegin();
    (*(it_rsch->second.begin()))->printEvolution(false, -1);
  } else {
    if (minSize > 1)
     return false;
    print();
  }
  
  return true;
}

/**
 * @fn bool Community::setMembers(std::tr1::shared_ptr< Community > other) const
 * Set the members of this community equal to the members of a given community.
 * @param other Community to set members equal to
 * @return True if all members are in the graph and were copied successfully.
 */
bool Community::setMembers(std::tr1::shared_ptr< Community > other) const
{
  for (it_m = other->members.begin(); it_m != other->members.end(); it_m++){
    if (G->hasVertex(*it_m)){
      this->members.insert(*it_m);
    } else {
      return false;
    }
  }

  return true;
}

double Community::Win() { return w_in; }
double Community::Wout() { return w_out; }

/**
 *@fn double Community::calcWin()
 *Simple 'get' function
 *@return Sum of weights of positive weighted edges with both endpoints in the community.
 */
double Community::calcWin(){
  w_in = 0;
  
  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();

    while (it_e != n->end()){      
      if ((it_e->second > 0) && (members.find(it_e->first) != members.end())){
	w_in += it_e->second;
      }

      ++it_e;
    }
  }

  w_in = w_in / (G->iDenFactor());
  
  return (w_in);
}

double Community::BinWin(){
  double result = 0;
  
  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second > 0) && (members.find(it_e->first) != members.end()))
	result += 1.0;

      ++it_e;
    }
  }
  
  return (result / (G->iDenFactor()));
}

/**
 *@fn double Community::NegWin()
 *Simple 'get' function
 *@return Sum of weights of negative edges with both endpoints in the community.
 */
double Community::NegWin(){
  double result = 0;
  
  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second < 0) && (members.find(it_e->first) != members.end()))
	result += it_e->second;

      ++it_e;
    }
  }
  
  return (result / (G->iDenFactor()));
}

double Community::BinNegWin(){
  double result = 0;
  
  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second < 0) && (members.find(it_e->first) != members.end()))
	result += -1;

      ++it_e;
    }
  }
  
  return (result / (G->iDenFactor()));
}

/**
 *@fn double Community::calcWout()
 *Simple 'get' function
 *@return Sum of weights of positive edges with only one endpoint in the community.
 */
double Community::calcWout(){
  w_out = 0;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second > 0) && (members.find(it_e->first) == members.end()))
	w_out += it_e->second;

      ++it_e;
    }
  }
  
  return w_out;
}

double Community::BinWout(){
  double result = 0;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second > 0) && (members.find(it_e->first) == members.end()))
	result += 1;

      ++it_e;
    }
  }
  
  return result;
}

/**
 *@fn double Community::NegWout()
 *Simple 'get' function
 *@return Sum of weights of negative edges with only one endpoint in the community.
 */
double Community::NegWout(){
  double result = 0;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second > 0) && (members.find(it_e->first) == members.end()))
	result += it_e->second;

      ++it_e;
    }
  }
  
  return result;
}

double Community::BinNegWout(){
  double result = 0;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    if (n == NULL)
      continue;
    
    Graph::edge_iterator it_e = n->begin();
    while (it_e != n->end()){
      if ((it_e->second > 0) && (members.find(it_e->first) == members.end()))
	result += -1;

      ++it_e;
    }
  }
  
  return result;
}

/**
 *@fn double Community::InDeg()
 *Simple 'get' function
 *@param node Vertex (not a member of the community usually) of interest
 *@return Sum of weights of positive edges that connect node to a vertex in the community.
 */
double Community::InDeg(Graph::vertex node){
  double result = 0;
  
  Graph::edgelist* n = G->getNeighborhood(node);
  if (n != NULL){
    for (Graph::edge_iterator it_e = n->begin(); it_e != n->end(); it_e++){
      if ((members.find(it_e->first) != members.end()) && (it_e->second > 0))
	result += it_e->second;
    }
  }

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* mn = G->getNeighborhood(*it_m);
    if (mn == NULL)
      continue;
    
    Graph::edge_iterator it_e = mn->find(node);
    if ((it_e != mn->end()) && (it_e->second > 0)){
      result += it_e->second;
    }
  }

  return (result / G->iDenFactor());
}

pair < double, double > Community::InOutDeg(Graph::vertex node){
  double indeg = 0, outdeg = 0;

  Graph::edgelist* n = G->getNeighborhood(node);
  if (n != NULL){
    for ( Graph::edge_iterator it_e = n->begin(); it_e != n->end(); it_e++){
      if ( ( members.find(it_e->first) != members.end() ) && (it_e->second > 0) )
	indeg += it_e->second;
      else
	outdeg += it_e->second;
    }
  }

  return pair <double, double> (indeg, outdeg);
}

/**
 *@fn double Community::NegInDeg()
 *Simple 'get' function
 *@param node Vertex (not a member of the community usually) of interest
 *@return Sum of weights of negative edges that connect node to a vertex in the community.
 */
double Community::NegInDeg(Graph::vertex node){
  double result = 0;
  
  Graph::edgelist* n = G->getNeighborhood(node);
  if (n != NULL){
    for (Graph::edge_iterator it_e = n->begin(); it_e != n->end(); it_e++){
      if ((members.find(it_e->first) != members.end()) && (it_e->second > 0))
	result += it_e->second;
    }
  }

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* mn = G->getNeighborhood(*it_m);
    if (mn == NULL)
      continue;
   
    Graph::edge_iterator it_e = mn->find(node);
    if ((it_e != mn->end()) && (it_e->second < 0)){
      result += it_e->second;
    }
  }

  return result;
}

/**
 *@fn std::map< Graph::vertex, double, cmp_tr1_string > Community::getNeighborhood()
 * Retrieve the fringe vertices of the community.
 *
 *@return Map of vertices not in the community that are connected to a node in the community to a weight of one such edge
 *   (these weights aren't deterministic and shouldn't be used)
 */
std::map< Graph::vertex, double, cmp_tr1_string > Community::getNeighborhood(){
  map < Graph::vertex, double, cmp_tr1_string > result;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    for (Graph::edge_iterator it_e = n->begin(); it_e != n->end(); it_e++){
      if (members.find(it_e->first) == members.end())
	result.insert(*it_e);
    }
  }
    
  return result;
}

/**
 *@fn std::map< Graph::vertex, double, cmp_tr1_string > Community::getMembers()
 * See return explanation
 * @return Map of vertices in the community a weight of one of it's internal edges
 *   (these weights aren't deterministic and shouldn't be used)
 */
std::map< Graph::vertex, double, cmp_tr1_string > Community::getMembers(){
  map < Graph::vertex, double, cmp_tr1_string > result;

  for (it_m = members.begin(); it_m != members.end(); it_m++){
    Graph::edgelist* n = G->getNeighborhood(*it_m);
    for (Graph::edge_iterator it_e = n->begin(); it_e != n->end(); it_e++){
      if (members.find(it_e->first) != members.end())
	result.insert(*it_e);
    }
  }
    
  return result;
}

std::tr1::shared_ptr < Community > Community::nextInEvo(){
  if ((parents.size() > 0))
    return std::tr1::shared_ptr < Community > ();

  if (children.size() > 0){
    std::map < int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > >::reverse_iterator it_rchain = children.rbegin();
    std::map < double, std::set < std::tr1::shared_ptr< Community > > >::reverse_iterator it_rsch = it_rchain->second.rbegin();
    return (*(it_rsch->second.begin()));
  } else {
    return std::tr1::shared_ptr < Community > ();
  }
}

/**
 *@fn double Jaccard(Community* A, Community* B)
 *
 * Calculate the Jaccard Index - the number of similar members divided by the total number of unique members of both communities. 
 *
 * @param A Community 1
 * @param B Community 2
 * @return Jaccard index between A and B
 */
double Jaccard(Community* A, Community* B){
  double similar = 0;

  set < Graph::vertex >::iterator it_m;
  for (it_m = A->first(); it_m != A->last(); it_m++){
    Graph::vertex tmp(new string(**it_m));

    if (B->hasMember(tmp)){
      ++similar;
    }
  }

  double denominator = A->size() + B->size() - similar;

  return similar / denominator;
}

/**
 *@fn double Jaccard(const std::tr1::shared_ptr< Community > A, const std::tr1::shared_ptr< Community > B)
 * Helpful overload for working with shared pointers.

 * @param A Community 1
 * @param B Community 2
 * @return Jaccard between A and B
 */
double Jaccard(const std::tr1::shared_ptr< Community > A, const std::tr1::shared_ptr< Community > B){
  return Jaccard(&(*A), &(*B));
}

/**
 *@fn double Intersection(Community* A, Community* B)
 *
 * Calculate the Number of similar members between A and B
 *
 * @param A Community 1
 * @param B Community 2
 * @return Number of similar members between A and B
 */
double Intersection(Community* A, Community* B){
  double similar = 0;

  set < Graph::vertex >::iterator it_m;
  for (it_m = A->first(); it_m != A->last(); it_m++){
    Graph::vertex tmp(new string(**it_m));

    if (B->hasMember(tmp)){
      ++similar;
    }
  }

  return similar;
}

/**
 *@fn double Intersection(const std::tr1::shared_ptr< Community > A, const std::tr1::shared_ptr< Community > B)
 * Helpful overload for working with shared pointers.

 * @param A Community 1
 * @param B Community 2
 * @return Number of similar members between A and B
 */
double Intersection(const std::tr1::shared_ptr< Community > A, const std::tr1::shared_ptr< Community > B){
  return Intersection(&(*A), &(*B));
}
