#ifndef COMMUNITY_JT
#define COMMUNITY_JT

#include "Graph.h" 

/**
 * @class Community 
 *
 * Representation of a community in a network
 */
class Community{
public:
  
  static std::tr1::shared_ptr<Community> ptrCommunity(const std::set< Graph::vertex, cmp_tr1_string > nodes, std::tr1::shared_ptr < Graph > G){
    std::tr1::shared_ptr< Community > result(new Community(nodes, G));
    return result;
  }
  
  /**
   * @fn Community()
   * 
   * Empty Constructor - Null out child pointer
   */
 explicit Community(std::tr1::shared_ptr < Graph > Gr):G(Gr){};
  
  /**
   * @fn Community(const std::set< std::tr1::shared_ptr< const std::string >, cmp_tr1_string > nodes)
   * @param c Set of vertices to initialize this community as
   */
  explicit Community(const std::set< Graph::vertex, cmp_tr1_string > nodes, std::tr1::shared_ptr < Graph > Gr);
  
  /**
   * @fn Community(const Community& other)
   * @param other Community to copy
   * 
   * Copy Constructor
   */
  Community(const Community& other);
  
  /**
   * @fn Community(const Community* other)
   * @param other Community to copy
   * 
   * Copy Constructor
   */
  Community(const Community* other);
  
  /**
   * @fn ~Community()
   * 
   * Empty Destructor - all dynamic memory should be smart pointers
   */
  ~Community(){}
  
  /**
   * @fn bool operator<(const Community& other)
   * @param other Community to compare to
   * 
   * Function to use in order to compare communities (for use in containers)
   * 
   * @return True if this community alphabetically preceeds the other
   * 		False otherwise
   */
  bool operator<(const Community& other);
  
  /**
   * @fn bool hasMember(std::tr1::shared_ptr<std::string> node)
   * @param node A vertex
   * 
   * @return True if the given node is in the community
   * 		False otherwise
   */
  bool hasMember(std::tr1::shared_ptr<std::string> node);
  bool hasMember(std::tr1::shared_ptr<const std::string> node);

  /**
   * @fn void addMember(std::tr1::shared_ptr<std::string> node)
   * @param node A vertex
   * 
   * Inserts the given node into the community
   * 	There is no connectivity / error checking done by this function at this time
   */
  bool addMember(std::tr1::shared_ptr<const std::string> node);
  
  /**
   * @fn void removeMember(std::tr1::shared_ptr<std::string> node)
   * @param node A vertex
   * 
   * Removes the given node from the community (if it was a member previously)
   * 	There is no connectivity / error checking done by this function at this time
   */
  bool removeMember(std::tr1::shared_ptr<const std::string> node);
  
  /**
   * @fn int size()
   * @return Number of members in the community
   */
  int size();
  
  /**
   * @fn std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator first()
   * @return Iterator to the first member of this community
   */
  std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator first();
  std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator first() const;
  /**
   * @fn std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator last()
   * @return Iterator to the last member of this community
   */
  std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator last();
  std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::const_iterator last() const;
  /**
   * @fn print()
   * @fn print() const
   * 
   * Send member list to standard output on a single line (separated by commas)
   */
  void print();
  void print() const;

  void print(std::ostream& fout);
  void print(std::ostream& fout) const;
    
  /**
   * @fn void addChild(int len, double str, std::tr1::shared_ptr<Community> other)
   * @param len Length of the evolution chain (including the current community)
   * @param str Strength of tie with the next community in the chain
   * @param other Community to provide the next link in the chain
   * 
   * Add other community as a potential next evolution link, tracking the evolution length and the strength of the tie
   */
  void addChild(int len, double str, std::tr1::shared_ptr<Community> other);
  
  /**
   * @fn void addParent(int len, double str, std::tr1::shared_ptr<Community> other)
   * @param len Length of the evolution chain (including the current community)
   * @param str Strength of tie with the next community in the chain
   * @param other Community to provide the next link in the chain
   * 
   * Add other community as a potential parent evolution link, tracking the evolution length and the strength of the tie
   */
  void addParent(int len, double str, std::tr1::shared_ptr<Community> other);

  /**
   * @fn int longest()
   * 
   * @return The length of the longest evolution starting from this community.
   */
  int longest();
  
  /**
   * @fn int preLongest()
   * 
   * @return The length of the longest evolution ending at this community.
   */
  int preLongest();
  
  /**
   * @fn void printEvolution(bool careAboutParents, const int& minSize)
   * @param careAboutParents Flag to distinguish whether or not to print only evolutions that begin with this community or all
   * 	partial evolutions that pass through this community
   * @param minSize Minimum remaining length of an evolution required to print it out
   * 
   * Will print one community per line.
   * 
   */
  bool printEvolution(bool careAboutParents, const int& minSize);

  std::tr1::shared_ptr< Community > nextInEvo();

  double Win();
  double calcWin();
  double BinWin();
  double NegWin();
  double BinNegWin();
  double Wout();
  double calcWout();
  double BinWout();
  double NegWout();
  double BinNegWout();
  double InDeg(Graph::vertex node);
  pair < double, double > InOutDeg(Graph::vertex node);
  double NegInDeg(Graph::vertex node);
  
  map < Graph::vertex, double, cmp_tr1_string > getNeighborhood();
  map < Graph::vertex, double, cmp_tr1_string > getMembers();

  map < Graph::vertex, int, cmp_tr1_string > pVector( const int& maxSteps );
  
  bool setMembers(std::tr1::shared_ptr<Community> other) const;
  
  /**
   * @struct cmp_comm
   * Comparison structure to keep Communities alphabetically ordered in STL containers
   */
  struct cmp_comm{
    bool operator() (const std::tr1::shared_ptr<Community> A, const std::tr1::shared_ptr<Community> B);
  };
private:
  double w_in, w_out;

  mutable std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string > members; // The nodes of the community
  std::tr1::shared_ptr < Graph > G;

  std::map < int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > > parents, children; // Evolution chain links
  std::map < int, std::map < double, std::set < std::tr1::shared_ptr< Community > > > >::iterator it_chain;
  std::map < double, std::set < std::tr1::shared_ptr< Community > > >::iterator it_sch;
  
  mutable std::set < std::tr1::shared_ptr<const std::string>, cmp_tr1_string >::iterator it_m; // Helper iterator for members list
};

/*Function to find the Jaccard Index between two communities*/
double Jaccard(Community* A, Community* B);
double Jaccard(const std::tr1::shared_ptr < Community > A, const std::tr1::shared_ptr < Community > B);

/*Function to find Intersection between two communities (similar members)*/
double Intersection(Community* A, Community* B);
double Intersection(const std::tr1::shared_ptr < Community > A, const std::tr1::shared_ptr < Community > B);

typedef std::tr1::shared_ptr < Community > CommunityPtr;

#endif
