#include "Density.h"

/**
 * @fn std::tr1::shared_ptr< Density > Density::Create(char type, std::tr1::shared_ptr<Parameters> p)
 * @param type Type of density structure to create
 * @param p Command line parameters - to be used in some initializations
 * 
 * @return Density structure of specified type
 */
std::tr1::shared_ptr< Density > Density::Create(char type, std::tr1::shared_ptr<Parameters> p)
{
  double lambda, a, b, c, d;
  p->set<double>(&lambda, "l", 0.5);

  p->set<double>(&a, "alpha", 1);
  p->set<double>(&b, "beta", 1);
  p->set<double>(&c, "gamma", 1);
  p->set<double>(&d, "delta", 1);

  std::tr1::shared_ptr< Density > complex(new ComplexDensity(lambda));
  std::tr1::shared_ptr< Density > sgnd(new SignedDensity(a, b, c, d));
  
  switch(type){
	case 's':
	  return sgnd;
	default:
	  return complex;
  }
}

/**
 * @fn bool AddIfValid(std::tr1::shared_ptr < Network > net, std::tr1::shared_ptr<std::string> node)
 * @param net Network to consider
 * @param node Vertex to try to add to Community
 *
 * Will check whether adding the given vertex will increase the density measure for
 * the saved community or not. If so, the vertex is added to the community.
 * Only vertices with a positive connection to at least one vertex in the community
 * are considered.
 *
 * @return true if vertex was added to community, false otherwise
 */
bool ComplexDensity::AddIfValid(std::tr1::shared_ptr < const std::string> node, std::tr1::shared_ptr < Community > c, std::tr1::shared_ptr< Graph > g){
  double c_win = c->Win(), c_wout = c->Wout();
  double Vin = c->InDeg(node);
  double Vout = g->PosWeiDegree(node) - Vin;
  
  double left = (1 - lambda) * (c_win / (c_win + c_wout));
  double right;
  if (c->size() == 1)
    right = 0;
  else
    right = lambda * (c_win / ((c->size() * (c->size() - 1)) / 2.0));
  
  double nleft = (1 - lambda) * ((c_win + Vin) / (c_win + c_wout + Vout));
  double nright = lambda * ((c_win + Vin) / ((c->size() * (c->size() + 1)) / 2.0));

  if (left + right <= nleft + nright){
    c->addMember(node);    
    return true;
  }

  return false;
}
