#ifndef DENSITY_JT
#define DENSITY_JT

#include "Network.h"
#include <limits>
#include "../Toolbox/Util/Parameters.h"

/**
 * @class Density
 * 
 * Base structure to help calculate 'goodness' values for potential communities
 */
class Density{
public:
  virtual double calc(std::tr1::shared_ptr< Community > c) = 0; //Calculate the density measure
  virtual double bincalc(std::tr1::shared_ptr< Community > c) {return 1;}; //Calculate the density measure
  virtual bool AddIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g) = 0; //Add a new vertex to the communitiy if the density increases
  virtual bool RemoveIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g) = 0; // Remove a member if doing so increases the density

  static std::tr1::shared_ptr<Density> Create(char type, std::tr1::shared_ptr<Parameters> p); //Factory function
 
private:
  
};

/**
 *@class SignedDensity: public Density
 *
 *  Density calculation better suited to handling networks with signed edge weights. 
 */
class SignedDensity: public Density{
 public:

  /**
   * @fn SignedDensity()
   *
   *   Empty constructor - no extra weighting of edges
   */
 SignedDensity():alpha(1), beta(1), gamma(1), delta(1){}; 

  /**
   * SignedDensity(double a, double b, double c, double d)                                                                        
   *                                                                                                                                      * A weighting of edges. 
   * @param a Positive internal edge multiplier
   * @param b Negative internal edges
   * @param c Positive external edges
   * @param d Negative external edges
   */
 SignedDensity(double a, double b, double c, double d):alpha(a), beta(b), gamma(c), delta(d){};

  /**
   * @fn double calc(std::tr1::shared_ptr<Community> c)
   *
   * @param c Community to calculate density for
   * @return The density for signed networks - ((Win^+ + Win^-) - (Wout^+ + Wout^-)) / Size
   */
 double calc(std::tr1::shared_ptr<Community> c){
    if (c->size() < 1){
      return numeric_limits<double>::min();
    }

    return (((alpha * c->Win()) + (beta * c->NegWin())) - ((gamma * c->Wout()) + (delta * c->NegWout()))) / double(c->size());
  }

 
 /**
  *@fn bool AddIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g)
  *   If adding the given vertex to the given community will increase the community's density, do it.
  *
  * @param node Vertex to try to add to community
  * @param c Community to try and add vertex to
  * @pararm g Graph structure to consider for density calculation
  *
  */
  bool AddIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g){
    //Get all pertinent values
    double Vin = c->InDeg(node);
    double Vnin = c->NegInDeg(node);
    double Vout = g->PosWeiDegree(node) - Vin;
    double Vnout = g->NegWeiDegree(node) - Vnin;

    double win = c->Win();
    double nwin = c->NegWin();
    double wout = c->Wout();
    double nwout = c->NegWout();

    double s = double(c->size());

    //Check the difference in density. If positive, add vertex
    if (((((alpha * (win + Vin)) + beta * (nwin + Vnin)) - ((gamma * (wout - Vin + Vout)) + (delta * (nwout - Vnin + Vnout)))) / (s + 1)) > ((((alpha * win) + (beta * nwin)) - ((gamma * wout) + (delta * nwout))) / s)){
      c->addMember(node);
      return true;
    }
    
    return false;
  }

  
 /**
  *@fn bool RemoveIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g)
  *   If removing the given vertex to the given community will increase the community's density, do it.
  *
  * @param node Vertex to try to remove to community
  * @param c Community to try and remove vertex to
  * @pararm g Graph structure to consider for density calculation
  *
  */
  bool RemoveIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g){
    //Get values
    double Vin = c->InDeg(node);
    double Vnin = c->NegInDeg(node);
    double Vout = g->PosWeiDegree(node) - Vin;
    double Vnout = g->NegWeiDegree(node) - Vnin;

    double win = c->Win();
    double nwin = c->NegWin();
    double wout = c->Wout();
    double nwout = c->NegWout();

    double s = double(c->size());
    
    if (s == 1)
      return false;

    //cerr << "Remove Check : " << *node << " " << Vin << " " << Vout << " " << win << " " << wout << " " << s << endl;

    //Check difference and remove vertex if appropriate
    
    double newDens = (((alpha * (win - Vin)) + (beta * (nwin - Vnin))) - ((gamma * (wout - Vout + Vin)) + (delta * (nwout -Vnout + Vnin)))) / (s - 1);

    double oldDens = (((alpha * win) + (beta * nwin)) - ((gamma * wout) + (delta * nwout))) / s;

    if (newDens > oldDens){
      //cerr << "Remove : " << *node << endl;
      c->removeMember(node);
      return true;
    }

    return false;
  }

 private:
  double alpha, beta, gamma, delta; // Weights for positive internal, negative internal, positive external and negative external edges

};

/**
 * @class ComplexDensity
 * 
 * Structure to calculate density for use in positive weighted networks
 */
class ComplexDensity: public Density{
public:
  
  /**
   * @fn ComplexDensity(double l):lambda(l)
   * 
   * Simple Constructor
   */
  ComplexDensity(double l):lambda(l){
    
  };
  
  
  /**
   * @fn double calc(std::tr1::shared_ptr<Network> net, std::tr1::shared_ptr<Community> com)
   * @param net Network to consider for density measure
   * @param com Community to calculate density for
   * 
   * This function will replace and save the community in this structure for later use
   * 
   * @return Density or 'quality' measure of community within a positive-weighted network
   */
  double calc(std::tr1::shared_ptr< Community > c){
    if (c->size() <= 1)
      return 0;
   
    double left = (1 - lambda) * (c->Win() / (c->Win() + c->Wout()));
    double right = lambda * (c->Win() / ((c->size() * (c->size() - 1)) / 2.0)); 

    return left + right;
  };
 
  double bincalc(std::tr1::shared_ptr< Community > c){
    if (c->size() <= 1)
      return 0;
   
    double left = (1 - lambda) * (c->BinWin() / (c->BinWin() + c->BinWout()));
    double right = lambda * (c->BinWin() / ((c->size() * (c->size() - 1)) / 2.0)); 

    return left + right;
  };

  /**
   * @fn bool AddIfValid(std::tr1::shared_ptr<Network> net, std::tr1::shared_ptr<std::string> node)
   * @param net Network to consider
   * @param node Vertex to try to add to Community
   * 
   * Will check whether adding the given vertex will increase the density measure for 
   * 	the saved community or not. If so, the vertex is added to the community.
   * Only vertices with a positive connection to at least one vertex in the communtiy
   * 	are considered
   *  
   * @return true if vertex was added to community, false otherwise
   * 
   */
  bool AddIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g);
  
  /**
   * @fn bool RemoveIfValid(std::tr1::shared_ptr<Network> net, std::tr1::shared_ptr<std::string> node)
   * @param net Network to consider
   * @param node Vertex to try to remove to Community
   * 
   * Will check whether removing the given vertex will increase the density measure for 
   * 	the saved community or not. If so, the vertex is removed to the community.
   * 
   * @return true if vertex was removed from community, false otherwise
   * 
   */
  bool RemoveIfValid(std::tr1::shared_ptr<const std::string> node, std::tr1::shared_ptr< Community > c, std::tr1::shared_ptr< Graph > g){
      if (c->size() <= 1)
	return false;

      double Vin = c->InDeg(node);
      double Vout = g->PosWeiDegree(node) - Vin;
      
      double left = (1 - lambda) * (c->Win() / (c->Win() + c->Wout()));
      double right = lambda * (c->Win() / ((c->size() * (c->size() - 1)) / 2.0)); 
      
      double nleft = (1 - lambda) * ((c->Win() - Vin) / (c->Win() + c->Wout() - Vout));
      
      double nright;
      if (c->size() == 2)
	nright = 0;
      else
	nright = lambda * ((c->Win() - Vin) / (((c->size() - 1) * (c->size() - 2)) / 2.0)); 
      
      if (left + right < nleft + nright){
	c->removeMember(node);
	return true;
      }
      
      return false;
  }
  
private:
  double lambda; //Tunable parameter to change focus on internal edge density.
 };

 #endif
