#include "Graph.h"
/**
 * @fn LoadNetwork(const std::string& filename, const std::string& delimiter, const double& weightThreshold)
 *
 * Load a network structure from a file.
 *
 *@param filename File to look at
 *@param delimiter Character used to delimit words on a line
 *@param weightThreshold Minimum allowable edge weight
 */
void DirectedGraph::LoadStructure(const std::string& filename, const std::string& delimiter, const double& weightThreshold, const bool positive)
{
  std::ifstream fin;
  
  if (!openFile(&fin, filename)){
    cerr << "Could not open file " << filename << endl;
    exit(2);
  }
  
  std::vector < std::string > fields;
  
  while (fline_tr(&fin, &fields, delimiter)){
    double weight = str_to<double>(fields[2]);

    if (positive && (weight <= 0))
      continue;

    //Check Weight Threhsold
    if (weight > 0 && weight < weightThreshold)
      continue;
    if (weight <= 0 && -weight < weightThreshold)
      continue;
    
    vertex currentBase(new std::string(fields[0]));
    vertex currentTarget(new std::string(fields[1]));
    
    vertex_iterator it_n;
    edge_iterator it_e;
    
    //Add Base->Target
    if ((it_n = G.find(currentBase)) != G.end()){
      if ((it_e = it_n->second.find(currentTarget)) != it_n->second.end()){
	it_e->second += weight;
      } else {
	it_n->second.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentTarget, weight));
      }
    } else {
      edgelist temp;
      temp.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentTarget, weight));
      G.insert(make_pair<std::tr1::shared_ptr<const std::string>, map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string > > (currentBase, temp));
    }
  }
}

/**
 *@fn void DirectedGraph::PrintStructure(ostream& fout)
 *
 *Prints out the graph structure in an edge list format, each line being (source, target, weight).
 *
 *@param fout Stream to print to
 */
void DirectedGraph::PrintStructure(ostream& fout){
  vertex_iterator it_n;
  edge_iterator it_e;
    
  for (it_n = G.begin(); it_n != G.end(); it_n++){
    for (it_e = it_n->second.begin(); it_e != it_n->second.end(); it_e++){
      fout << *(it_n->first) << ", " << *(it_e->first) << ", " << it_e->second << endl;
    }
  }
}

/**
 *@fn unsigned int DirectedGraph::nvertices()
 *
 * Overloaded to compensate for the difference between directed and undirected network structure representation.
 *
 *@return Number of vertices in the network
 */
unsigned int DirectedGraph::nvertices(){
  vertex_iterator it_n;
  edge_iterator it_e;

  set < vertex, cmp_vertex > nodes;
  for ( it_n = G.begin(); it_n != G.end(); it_n++ ){
    nodes.insert(it_n->first);
    for ( it_e = it_n->second.begin(); it_e != it_n->second.end(); it_e++){
      nodes.insert(it_e->first);
    }
  }

  return nodes.size();
}
