#include "Graph.h"

/**
 * @fn printEdgeList(Graph::edgelist edges)
 *
 * Prints a list of edges [target, weight] to the error stream
 *
 * @param edges List to print
 */
void printEdgeList(Graph::edgelist edges){
  Graph::edge_iterator it_e = edges.begin();
  for (; it_e != edges.end(); it_e++){
    cerr << *(it_e->first) << " " << it_e->second << endl;
  }
};

/**
 *@fn void printEdgeSet(std::map < Graph::vertex, std::set < Graph::edge_iterator, Graph::multi_cmp_edge_iterator >, cmp_tr1_string > edges)
 *
 *  Prints a set of edge lists (source target weight) to error stream.
 *
 * @param edges List to print
 */
void printEdgeSet(std::map < Graph::vertex, std::set < Graph::edge_iterator, Graph::multi_cmp_edge_iterator >, cmp_tr1_string > edges){
  std::map < Graph::vertex, std::set < Graph::edge_iterator, Graph::multi_cmp_edge_iterator >, cmp_tr1_string >::iterator it_v;
  set < Graph::edge_iterator, Graph::multi_cmp_edge_iterator >::iterator it_e;
  
  for (it_v = edges.begin(); it_v != edges.end(); it_v++){
    for (it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
      cerr << *(it_v->first) << " " << *((*it_e)->first) << " " << (*it_e)->second << endl;
    }
  }
};

/**
* @fn Graph::Graph()
* 	Constructor
*/
Graph::Graph(){
  srand(time(NULL));
}

/**
 * @fn int Graph::nedges()
 * 
 * 	Counts the number of raw edges held in G.
 */
unsigned int Graph::nedges(){
  unsigned int result = 0;
  
  for (vertex_iterator it_v = G.begin(); it_v != G.end(); it_v++){
    for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
	++result;
    }
  }
  
  return result;
}

/**
 * @fn unsigned int Graph::nvertices()
 * 	
 * 	Counts number of raw vertices in G.
 */
unsigned int Graph::nvertices(){ 
  return G.size();
}

/**
 * @fn edgelist getNeighborhood(vertex node)
 * 	Returns list of vertices connected to 'node' as seen in G.
 */

Graph::edgelist* Graph::getNeighborhood(vertex node){
  vertex_iterator it_v = G.find(node);
  
  if (it_v == G.end())
    return NULL;
  else
    return &(it_v->second);
}

/**
 * @fn double Graph::Degree(Graph::vertex node, double threshold)
 *
 * Calculate the degree of a node while only considering edges with a (absolute value) weight above the threshold.
 *
 * @param node Vertex of interest
 * @param threshold Lowest weight to consider on an edge
 * @return Number of edges with weight higher than the threshold with the given node as an endpoint
 */
double Graph::Degree(Graph::vertex node, double threshold){
  double result = 0;
  
  vertex_iterator it_v = G.find(node);

  if (it_v == G.end())
    return 0;

  for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
    if (((it_e->second > 0) && (it_e->second >= threshold)) || ((it_e->second < 0) && (it_e->second <= -threshold)) )
      ++result;
  }
  
  return result;
}


/**
 *@fn double Graph::PosDegree(Graph::vertex node, double threshold)
 *
 * @param node Vertex of interest
 * @param threshold Lowest weight to consider on an edge
 * @return Number of positively weighted edges with weight higher than the threshold with the given node as an endpoint
 */
double Graph::PosDegree(Graph::vertex node, double threshold){
  double result = 0;
  
  vertex_iterator it_v = G.find(node);
  
  if (it_v == G.end())
    return 0;

  for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
    if ((it_e->second > 0) && (it_e->second >= threshold))
      ++result;
  }
  
  return result;
}

/**
 *@fn double Graph::NegDegree(Graph::vertex node, double threshold)
 *
 * @param node Vertex of interest
 * @param threshold Lowest weight to consider on an edge
 * @return Number of negatively weighted edges with an absolute value of a weight higher than the threshold with the given node as an endpoint
 */
double Graph::NegDegree(Graph::vertex node, double threshold){
  double result = 0;
  
  vertex_iterator it_v = G.find(node);
  
  if (it_v == G.end())
    return 0;

  for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
    if ((it_e->second < 0) && (-it_e->second >= threshold))
      ++result;
  }
  
  return result;
}

/**
 *@fn double Graph::PosWeiDegree(Graph::vertex node, double threshold)
 *
 * @param node Vertex of interest
 * @param threshold Lowest weight to consider on an edge
 * @return Sum of weights of positively weighted edges with weight higher than the threshold with the given node as an endpoint
 */
double Graph::PosWeiDegree(Graph::vertex node, double threshold){
  double result = 0;
  
  vertex_iterator it_v = G.find(node);
  if (it_v == G.end())
    return 0;

  for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
    if (it_e->second >= threshold)
	result += it_e->second;
  }
  
  return result;
}

/**
 *@fn double Graph::NegWeiDegree(Graph::vertex node, double threshold)
 *
 * @param node Vertex of interest
 * @param threshold Lowest weight to consider on an edge
 * @return Sum of weights of negatively weighted edges with an absolute value of a weight higher than the threshold with the given node as an endpoint
 */
double Graph::NegWeiDegree(Graph::vertex node, double threshold){
  double result = 0;
  
  vertex_iterator it_v = G.find(node);
  
  if (it_v == G.end())
    return 0;
  
  for (edge_iterator it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
    if (it_e->second < -threshold)
	result += it_e->second;
  }
  
  return result;
}

/**
 *@fn std::tr1::shared_ptr< Graph > Graph::Create(char type)
 *
 * Basic factory function to create a graph that's either directed or not.
 *
 *@return Pointer to a graph structure of the requested type.
 */
std::tr1::shared_ptr< Graph > Graph::Create(char type)
{
  if (type == 'd'){
    std::tr1::shared_ptr< Graph > dir(new DirectedGraph());
    return dir;
  } else if (type == 'u') {
    std::tr1::shared_ptr< Graph > udir(new UndirectedGraph());
    return udir;
  }
  std::tr1::shared_ptr< Graph > udir(new DirectedGraph());
    return udir;
}

/**
 * @fn vector < Graph::graph > Graph::GetComponents()
 *
 * Break the graph into its connected components
 *
 * @TODO : test with directed networks
 * @return A list of connected components (each one a graph in and of itself)
 */
vector < Graph::graph > Graph::GetComponents(){
  vector < Graph::graph > result;
  set < std::tr1::shared_ptr<const string>, cmp_tr1_string > seen;

  for (Graph::vertex_iterator it_v = G.begin(); it_v != G.end(); it_v++){
    if (seen.find(it_v->first) != seen.end())
      continue;

    graph component;

    set < std::tr1::shared_ptr<const string>, cmp_tr1_string > toConsider;
    toConsider.insert(it_v->first);
    seen.insert(it_v->first);
    
    while (toConsider.size() > 0){
      std::tr1::shared_ptr<const string> nextVertex = *(toConsider.begin());
      toConsider.erase(toConsider.begin());

      vertex_iterator it_v2;
      if ((it_v2 = G.find(nextVertex)) != G.end()){
	component.insert(*it_v2);
	for (edge_iterator it_e = it_v2->second.begin(); it_e != it_v2->second.end(); it_e++){
	  if ((seen.find(it_e->first) == seen.end())){
	    toConsider.insert(it_e->first);
	    seen.insert(it_e->first);
	  } 
	}
      }
    }

    result.push_back(component);
  }

  return result;
}

/**
 *@fn double Graph::getWeight(Graph::vertex v1, Graph::vertex v2)
 *
 * Get the weight of the edge between v1 and v2 (if any)
 *
 *@param v1, v2 Nodes of interest
 *@return Weight of edge between v1 and v2
 */
double Graph::getWeight(Graph::vertex v1, Graph::vertex v2){
  vertex_iterator it_v;
  if ((it_v = G.find(v1)) != G.end()){
    edge_iterator it_e;
    if ((it_e = it_v->second.find(v2)) != it_v->second.end()){
      return it_e->second;
    }
  }
  
  return 0;
}

/**
 *@fn bool Graph::hasVertex(Graph::vertex v)
 * Check if the graph contains a vertex
 * @param vertex of interest
 * @return True if the vertex is in the graph.
 */
bool Graph::hasVertex(Graph::vertex v){
  if (G.find(v) != G.end())
    return true;

  return false;
}

/**
 *@fn void Graph::NormalizeWeights()
 * Force all weights in the network to be between -1 and 1 (0 and 1 for positive networks)
 *
 */
void Graph::NormalizeWeights(){
  double maxWeight = 0;
  vertex_iterator it_v;
  for (it_v = G.begin(); it_v != G.end(); it_v++){
    edge_iterator it_e;
    for (it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
      //Check for negative large values
      if ((it_e->second < 0) && (-it_e->second > maxWeight)){
	maxWeight = -it_e->second;
      } 

      //Check positive values
      if ((it_e->second > 0) && (it_e->second > maxWeight)){
	maxWeight = it_e->second;
      }
    }
  }

  //Reduce weights
  for (it_v = G.begin(); it_v != G.end(); it_v++){
    edge_iterator it_e;
    for (it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
      it_e->second /= maxWeight;
    }
  }
}

/**
 *
 *Takes a file of raw interaction data with each line being:
 *
 *   Fr     To      Timestamp     Extra Data
 *
 *Splits the data into time windows and constructs appropriate undirected networks.


To Do:
    Directed Network Creation
 *
 */
void Graph::RawToStructure(const string& filename, const string& delim, const Timestamp& begin, const Timestamp& window, const Timestamp& step, const string& outputPrefix){
  ifstream fin;

  //Read in the information - this could be changed so that 
  //    data is read in as a stream.
  if (!openFile(&fin, filename)){
    cerr << "Could not file raw file : " << filename << endl;
    exit(1);
  } else {
    vector < string > fields;
    multimap < Timestamp, pair < string, string > > Interactions;
    multimap < Timestamp, pair < string, string > >::iterator it_int;

    while (fline_tr(&fin, &fields, delim)){
      if (fields.size() < 3)
	continue;

      string FR = fields[0];
      string TO = fields[1];
      string DATE = fields[2];

      Timestamp T(DATE);

      if (!T.isValid())
	continue;

      Interactions.insert(pair< Timestamp, pair < string, string > > (T, pair<string, string> (TO, FR)));
    }
    
    fin.close();
    
    //Initialize timestamps
    Timestamp A(begin.getstr()), B(begin.getstr());
    B = B + window;

    A.print();
    B.print();

    int round = 0;
    
    map < string, map < string, double > > net;
    map < string, map < string, double > >::iterator it_n;
    map < string, double >::iterator it_ed;
    ofstream fout;

    //Go through information
    for (it_int = Interactions.begin(); it_int != Interactions.end(); it_int++){
      //If you have to, ignore the first couple
      if (it_int->first < A)
	continue;

      //If the other window is reached, print out what you've got
      while (it_int->first > B){
	fout.open((outputPrefix + to_str<int>(round)).c_str());

	for (it_n = net.begin(); it_n != net.end(); it_n++){
	  for (it_ed = it_n->second.begin(); it_ed != it_n->second.end(); it_ed++){
	    fout << it_n->first << ", " << it_ed->first << ", " << it_ed->second << endl;
	  }
	}

	fout.close();

	net.clear();

	A = A + step;
	B = B + step;

	A.print();
	B.print();

	while (it_int != Interactions.begin() && it_int->first > A){
	  --it_int;
	}

	if (it_int->first < A)
	  it_int++;

	++round;
      }

      //Add the edge (or increment the weight)
      string FR = it_int->second.first;
      string TO = it_int->second.second;
      
      if((it_n = net.find(FR)) != net.end()){
	if ((it_ed = it_n->second.find(TO)) != it_n->second.end()){
	  it_ed->second += 1;
	} else {
	  it_n->second.insert(pair<string, double>(TO, 1));
 	}
      } else {
	map < string, double > temp;
	temp.insert(pair<string, double>(TO, 1));
	net.insert(pair<string, map < string, double > >(FR, temp));
      }

      if((it_n = net.find(TO)) != net.end()){
	if ((it_ed = it_n->second.find(FR)) != it_n->second.end()){
	  it_ed->second += 1;
	} else {
	  it_n->second.insert(pair<string, double>(FR, 1));
	}
      } else {
	map < string, double > temp;
	temp.insert(pair<string, double>(FR, 1));
	net.insert(pair<string, map < string, double > >(TO, temp));
      }
    }

    //Print out the last iteration
    fout.open((outputPrefix + to_str<int>(round)).c_str());

	for (it_n = net.begin(); it_n != net.end(); it_n++){
	  for (it_ed = it_n->second.begin(); it_ed != it_n->second.end(); it_ed++){
	    fout << it_n->first << ", " << it_ed->first << ", " << it_ed->second << endl;
	  }
	}

	fout.close();
   
  }
}

/**
 *@fn vector < string > Graph::RandomCommunity(int size)
 *
 * Create a random connected set of vertices to serve as a community.
 *@param size Number of vertices in community
 * @return Vector of the names of the vertices
 */
vector < string > Graph::RandomCommunity(int size){
  set < vertex > emptySet;
  return RandomCommunity(size, emptySet);
}

/**
 *@fn vector < string > Graph::RandomCommunity(int size, set < vertex > used)
 *
 * Recursively called to create a random connected set of vertices to serve as a community. None of
 * the vertices in used can be chosen to expand the community.
 *
 *@param size Number of vertices in community
 *@param used Vertices already considered/eliminated from consideration
 *@return Vector of the names of the vertices
 */
vector <string> Graph::RandomCommunity(int size, set < vertex > used){
  vector < string > res;

  if ((size < 1) || used.size() == G.size())
    return res;

  set < vertex > randCom;

  int first = rand() % G.size();
  vertex_iterator v = G.begin();

  for (int i = 0; i < first; i++)
    v++;

  while (used.find(v->first) != used.end()){
    v++;
    if (v == G.end())
      v = G.begin();
  }
  
  randCom.insert(v->first);
  edgelist Neighbors;

  edge_iterator it_e, it_ee;

  for (it_ee = v->second.begin(); it_ee != v->second.end(); it_ee++){
    Neighbors.insert(pair<vertex, double>(it_ee->first, it_ee->second));
  }

  while (randCom.size() < (unsigned int)size){

    if (Neighbors.size() == 0){
      set < vertex >::iterator it_r;
      for (it_r = randCom.begin() ;it_r != randCom.end(); it_r++){
	used.insert(*it_r);
      }

      return RandomCommunity(size, used);
    }

    int next = rand() % Neighbors.size();
    it_e = Neighbors.begin();

    for (int k = 0; k < next; k++, it_e++);
    
    randCom.insert(it_e->first);

    v = G.find(it_e->first);
    Neighbors.erase(it_e);
    
    for (it_ee = v->second.begin(); it_ee != v->second.end(); it_ee++){
      if (randCom.find(it_ee->first) == randCom.end())
	Neighbors.insert(pair<vertex, double>(it_ee->first, it_ee->second));
    }
  }

  set < vertex >::iterator it_r;
  for (it_r = randCom.begin() ;it_r != randCom.end(); it_r++){
    res.push_back(**it_r);
  }

  return res;
}
