#ifndef GRAPH_JT
#define GRAPH_JT

#include <stdlib.h>
#include <tr1/memory>
#include <string>
#include <cstring>
#include <set>
#include <map>
#include <vector>
#include <iostream>
#include <fstream>

#include "../Toolbox/Util/IOX.h"
#include "../Toolbox/Util/StringEx.h"
#include "../Toolbox/Util/Parameters.h"
#include "../Toolbox/Time/Timestamp.h"

/**
 * @struct cmp_tr1_string
 * Comparison structure to use when storing tr1::shared_ptr<string> objects in 
 * 	STL containers - makes sure the pointers are stored alphabetically with respect to
 * 	the strings that they point to
 */
struct cmp_tr1_string{
  bool operator()(std::tr1::shared_ptr<const std::string> A, std::tr1::shared_ptr<const std::string> B){
    return (strcmp((*A).c_str(), (*B).c_str()) < 0);
  }
};

typedef struct cmp_tr1_string cmp_vertex;

/**
 * @class Graph
 * Provides lowest level structure for use in network analysis (definitions of vertices
 * 	edges, etc);
 */
class Graph{
public:  
  /**
  * Helpful typedefs
  */
  typedef std::map < std::tr1::shared_ptr<const std::string>, std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string >, cmp_tr1_string > graph;
  typedef std::map < std::tr1::shared_ptr<const std::string>, std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string >, cmp_tr1_string >::const_iterator const_vertex_iterator;
  typedef std::map < std::tr1::shared_ptr<const std::string>, std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string >, cmp_tr1_string >::iterator vertex_iterator;

  typedef std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string > edgelist;
  typedef std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string >::iterator edge_iterator;
  typedef std::map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string >::const_iterator const_edge_iterator;
  
  typedef std::tr1::shared_ptr<const std::string> vertex;

  /**
   * @struct cmp_edge_iterator
   * 	Comparison structure for edges iterators
   */
  struct cmp_edge_iterator{
    bool operator() (const edge_iterator& A, const edge_iterator& B){
      cmp_tr1_string ct;
      return ct(A->first, B->first);
    }
  };
  
  struct multi_cmp_edge_iterator{
    bool operator() (const edge_iterator& A, const edge_iterator& B){
      cmp_tr1_string ct;
      bool Z = ct(A->first, B->first);
      bool Y = ct(B->first, A->first);
      if (Z == Y){
	return A->second < B->second;
      }
      
      return Z;
    }
  };

  /**
   * @fn Graph()
   * 	Constructor
   */
  Graph();
  
  /**
   * @fn ~Graph()
   * 	Virtual destructor for multiple network types
   */
  virtual ~Graph(){};
  
  virtual void LoadStructure(const std::string& filename, const std::string& delimiter, const double& weightThreshold, const bool positive) = 0;
  virtual void PrintStructure(ostream& fout) = 0;
  virtual unsigned int nvertices();
  virtual unsigned int nedges();  
  edgelist* getNeighborhood(vertex node); 
  
  vertex_iterator vBegin(){return G.begin();};
  vertex_iterator vEnd(){return G.end();};

  double Degree(vertex node, double threshold = 0);
  double PosDegree(vertex node, double threshold = 0);
  double NegDegree(vertex node, double threshold = 0);
  double PosWeiDegree(vertex node, double threshold = 0);
  double NegWeiDegree(vertex node, double threshold = 0);
  double getWeight(vertex v1, vertex v2);

  void NormalizeWeights();
    
  bool hasVertex(vertex v);
  vector < string > RandomCommunity(int size);
  vector < string > RandomCommunity(int size, set < vertex > used);

  vector < graph > GetComponents();

  virtual double iDenFactor(){return 1.0;}
  
  static std::tr1::shared_ptr< Graph > Create(char type);
  static void RawToStructure(const string& filename, const string& delim, const Timestamp& begin, const Timestamp& window, const Timestamp& step, const string& outputPrefix);
protected:
   graph G;			//Holds the edge information
};

class DirectedGraph:public Graph{
public:
  ~DirectedGraph(){};
  void LoadStructure(const std::string& filename, const std::string& delimiter, const double& weightThreshold, const bool positive);
  void PrintStructure(ostream& fout);
  unsigned int nvertices();

private:
  
};

class UndirectedGraph:public Graph{
public:
  ~UndirectedGraph(){};
  void LoadStructure(const std::string& filename, const std::string& delimiter, const double& weightThreshold, const bool positive);
  void PrintStructure(ostream& fout);
  unsigned int nedges();
  double iDenFactor(){return 2.0;}

private:
  
};

void printEdgeList(Graph::edgelist edges);
void printEdgeSet(std::map < Graph::vertex, std::set < Graph::edge_iterator, Graph::multi_cmp_edge_iterator >, cmp_tr1_string > edges);
#endif
