#include "Network.h"

/**
 *@fn Network::Network(tr1::shared_ptr< Graph > g)
 *
 *Every network
 *
 *@param g Graph structure to initialize the network to.
 */
Network::Network(tr1::shared_ptr< Graph > g)
{
  G = g;
}


void Network::printCommunities(int minSize, ostream& fout)
{
  std::set < std::tr1::shared_ptr< Community > >::iterator it_c;
  for (it_c = communities.begin(); it_c != communities.end(); it_c++){
    if ((*it_c)->size() >= minSize)
      (*it_c)->print(fout);
  }
}

void Network::loadCommunities(const string& filename, const string& delimiter){
  ifstream fin;
  if (! openFile(&fin, filename.c_str()) ){
    cerr << "Could not open " << filename << endl;
    exit(3);
  }

  vector < string > fields;
  
  while (fline_tr(&fin, &fields, delimiter)){

    std::tr1::shared_ptr<Community> nextCommunity(new Community(G));

    for (int i = 0; i < fields.size(); i++){
      std::tr1::shared_ptr<const string> newMember (new string( replaceStrChar(fields[i], " ", '_') ));

      nextCommunity->addMember(newMember);
    }
    
    communities.insert(nextCommunity);

  }
}

/**
 * @fn void Network::calcParticipation()
 *
 * Constructs a map from each vertex to the communities that the vertex is a participant in.
 * Results are stored locally.
 */
void Network::calcParticipation(){

  std::set < std::tr1::shared_ptr< const std::string >, cmp_tr1_string >::const_iterator it_v;

  for (it_c = communities.begin(); it_c != communities.end(); it_c++){    //Each community
    for (it_v = (*it_c)->first(); it_v != (*it_c)->last(); it_v++){       //Each member
      /*Incorporate participation in map*/
      if ((it_p = participation.find(*it_v)) != participation.end()){     
	it_p->second.insert(*it_c);
      } else {
	std::set < std::tr1::shared_ptr< Community >, Community::cmp_comm > temp;
	temp.insert(*it_c);
	participation.insert(pair<std::tr1::shared_ptr< const std::string >, std::set < std::tr1::shared_ptr< Community >, Community::cmp_comm > >(*it_v, temp));
      }
    }
  }
  
}

void Network::getCurrentEvolutions(set < EvolutionPtr >& evolutions, const int& minSize){
  evolutions.clear();
  
  set < EvolutionPtr >::iterator it_s;
  for (it_s = E.begin(); it_s != E.end(); it_s++){
    if ((*it_s)->size() >= minSize){
      evolutions.insert(*it_s);
    }
  }
}

void Network::getLongestEvolutions(set < EvolutionPtr >& evolutions, const int& minSize){
  for (it_c = communities.begin(); it_c != communities.end(); it_c++){
    EvolutionPtr evo (new Evolution());
    
    CommunityPtr cp = *it_c;
    if (cp->longest() < minSize)
      continue;
    
    while (cp){
      evo->append(cp);
      cp = cp->nextInEvo();
    }
    
    evolutions.insert(evo);
  }
}

/**
 * @fn void Network::calcSimilarity(std::tr1::shared_ptr < Network > other, const double& threshold)
 *
 * Constructs linked lists within the Community modules that correspond to valid steps of evolution (as defined by some parameter).
 * The comparisons should be done in reverse order (compare step t to step t+1, then compare step t-1 to step t, etc) in order
 *  to get the tracking of evolution lengths accurate.
 */
void Network::calcSimilarity(std::tr1::shared_ptr < Network > other, const double& threshold){

  map < std::tr1::shared_ptr < const string >, std::set < std::tr1::shared_ptr< Community >, Community::cmp_comm >, cmp_tr1_string >::iterator it_o;

  map < std::tr1::shared_ptr < Community >, std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > pairs;
  map < std::tr1::shared_ptr < Community >, std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm >::iterator it_prs;

  std::set < std::tr1::shared_ptr < Community > , Community::cmp_comm >::iterator it_c2;
  
  /**
   *  First go through the communities and identify possible pairings ( one community from this timestep and one from the given 
   *    timestep that share at least 1 member.
   */
  for (it_o = other->pFirst(); it_o != other->pLast(); it_o++){
    if ((it_p = participation.find(it_o->first)) != participation.end()){
      for (it_c = it_p->second.begin(); it_c != it_p->second.end(); it_c++){
	if ((it_prs = pairs.find(*it_c)) == pairs.end()){
	  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm > tmp;
	  pairs.insert(pair <std::tr1::shared_ptr < Community >, std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm > > (*it_c, tmp));
	  it_prs = pairs.find(*it_c);
	}

	for (it_c2 = it_o->second.begin(); it_c2 != it_o->second.end(); it_c2++){
	  it_prs->second.insert(*it_c2);
	}
      }
    }
  }

  /**
   * If a connection is strong enough, make a note of it
   */
  for (it_prs = pairs.begin(); it_prs != pairs.end(); it_prs++){
    for (it_c = it_prs->second.begin(); it_c != it_prs->second.end(); it_c++){
      double strength = Jaccard(it_prs->first, *it_c);
            
      if (strength > threshold){
	it_prs->first->addChild((*it_c)->longest() + 1, strength,*it_c);
	(*it_c)->addParent(it_prs->first->preLongest() + 1 , strength,it_prs->first);
      }
    }
  }
}

void Network::printEvolutions(bool careAboutParents, int minLength){
  set < EvolutionPtr >::iterator it_s;
  for (it_s = E.begin(); it_s != E.end(); it_s++){
    if ((*it_s)->size() >= minLength)
      (*it_s)->Print();
  }
  
  /*cout << "---------------------------------------" << endl;
  for (it_c = communities.begin(); it_c != communities.end(); it_c++){
    if ((*it_c)->printEvolution(careAboutParents, minLength))
      cout << "---------------------------------------" << endl;
      }*/
}

double HXY ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B, double n ){
  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator it_a, it_b;

  set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_s, it_t;
  map < std::tr1::shared_ptr < Community >, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > check;
  map < std::tr1::shared_ptr < Community >, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm >::iterator it_c;

  A->calcParticipation();
  B->calcParticipation();

  check = CommunityMatch(A, B);

  double total = 0;

  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_cc;
  int minSize = B->getStructure()->nvertices();
  std::tr1::shared_ptr < Community > smallest;

  for (it_cc = B->cFirst(); it_cc != B->cLast(); it_cc++){
    if ((*it_cc)->size() < minSize){
      minSize = (*it_cc)->size();
      smallest = *it_cc;
    }
  }

  for (it_c = check.begin(); it_c != check.end(); it_c++){
    double current;
    double ca = it_c->first->size();

    double mn = (h(ca, n) + h(n - ca, n));

    for (it_s = it_c->second.begin(); it_s != it_c->second.end(); it_s++){
      double I = Intersection(it_c->first, *it_s);
      double cb = (*it_s)->size();

      //it_c->first->print();
      //(*it_s)->print();
      if ( ( current = H(n - cb - ca + I, cb - I, ca - I, I, n) ) < mn ){
	mn = current;
      }
      //cout << endl << endl;
      
    }
    
    //it_c->first->print();
    //cout << mn << endl << endl;

    total += mn;
  }

  return total;
}

double Hx( std::tr1::shared_ptr < Network > A, double n ){
   std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_c;

  double total = 0;

  //cerr << n << endl;
  
  for (it_c = A->cFirst(); it_c != A->cLast(); it_c++){
    total += h((*it_c)->size(), n) + h(n - (*it_c)->size(), n); 
    //cerr << total << endl;
  }

  return total;
}

double NMI (std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B){
  double n = Nodes(A, B);
  double hx = Hx(A, n), hy = Hx(B, n);
  double den = hy > hx ? hy : hx;
  double hxy = HXY(A, B, n);
  double hyx = HXY(B, A, n);

  if (den == 0)
    return 0;
 
  //cout << hx << " " << hxy << " " << hy << " " << hyx << endl;

  return (hx - hxy + hy - hyx) / (2 * den);
}

double Nodes(std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B){
  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator it_a, it_b;
  
  A->calcParticipation();
  B->calcParticipation();

  double n = B->participationSize();

  for (it_a = A->pFirst(); it_a != A->pLast(); it_a++){
    if (B->pMember(it_a->first) == B->pLast())
      ++n;
  }

  return n;
}

/**
 *Takes two networks, and returns a mapping from each community in A to a list of the communities in B that the first community shares at least one vertex with.
 */
std::map < std::tr1::shared_ptr < Community > , set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > CommunityMatch ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B){

  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator it_a, it_b;

  set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_s, it_t;
  map < std::tr1::shared_ptr < Community >, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > check;
  map < std::tr1::shared_ptr < Community >, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm >::iterator it_c;

  A->calcParticipation();
  B->calcParticipation();

  for (it_a = A->pFirst(); it_a != A->pLast(); it_a++){
    if ((it_b = B->pMember(it_a->first)) != B->pLast()){
      for (it_s = it_a->second.begin(); it_s != it_a->second.end(); it_s++){
	if ((it_c = check.find(*it_s)) == check.end()){
	  set < std::tr1::shared_ptr<Community>, Community::cmp_comm > tmp;
	  check.insert(pair<std::tr1::shared_ptr < Community >, set < std::tr1::shared_ptr < Community >, Community::cmp_comm > >(*it_s, tmp));
	  it_c = check.find(*it_s);
	}
	
	for (it_t = it_b->second.begin(); it_t != it_b->second.end(); it_t++){
	  it_c->second.insert(*it_t);
	}
      }
    }
  }

  return check;
}


double BestMove ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B ) {
  std::map < std::tr1::shared_ptr < Community > , set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > check = CommunityMatch( A, B );

  std::map < std::tr1::shared_ptr < Community > , set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm >::iterator it_c;
  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_s;

  double total = 0;

  for (it_c = check.begin(); it_c != check.end(); it_c++){
    double minDistance = it_c->first->size() + B->minCommunity();
    for (it_s = it_c->second.begin(); it_s != it_c->second.end(); it_s++){
      double intersection = Intersection(it_c->first, *it_s);

      double distance = (*it_s)->size() + it_c->first->size() - (2 * intersection);

      if ( distance < minDistance ){
	minDistance = distance;
      }
    }

    total += minDistance;
  }
  
  check = CommunityMatch( B, A );

  for (it_c = check.begin(); it_c != check.end(); it_c++){
    double minDistance = it_c->first->size() + A->minCommunity();
    for (it_s = it_c->second.begin(); it_s != it_c->second.end(); it_s++){
      double intersection = Intersection(it_c->first, *it_s);

      double distance = (*it_s)->size() + it_c->first->size() - (2 * intersection);

      if ( distance < minDistance ){
	minDistance = distance;
      }
    }

    total += minDistance;
  }

  return total / (double)(A->numCommunities() + B->numCommunities());
}

double FScore ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B, double threshold ) {
  std::map < std::tr1::shared_ptr < Community > , set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > check = CommunityMatch( A, B );

  std::map < std::tr1::shared_ptr < Community > , set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm >::iterator it_c;
  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator it_s;

  double matched = 0;

  for (it_c = check.begin(); it_c != check.end(); it_c++){
    for (it_s = it_c->second.begin(); it_s != it_c->second.end(); it_s++){
      if (Jaccard(it_c->first, *it_s) >  threshold){
	++matched;
	break;
      }
    }
  }

  check = CommunityMatch ( B, A );
  
  for (it_c = check.begin(); it_c != check.end(); it_c++){
    for (it_s = it_c->second.begin(); it_s != it_c->second.end(); it_s++){
      if (Jaccard(it_c->first, *it_s) >  threshold){
	++matched;
	break;
      }
    }
  }

  //cerr << matched << " " << A->numCommunities() << " " << B->numCommunities() << endl;

  return matched / (double)(A->numCommunities() + B->numCommunities());
}

double h ( double A, double N ){
  if (A == 0 || N == 0)
    return 0;

  return -A * (log(A/N) / log(2));
}

double H ( double a, double b, double c, double d, double n ){
  if (h(a, n) + h(d, n) < h(b, n) + h(c, n)){
    return h(c+d, n) + h(a+b, n);
  } else {
    return h(a, n) + h(b, n) + h(c, n) + h(d, n) - h(b + d, n) - h(a+c, n);
  }
}
