#ifndef NETWORK_JT
#define NETWORK_JT
#include "Evolution.h"

#include <stdlib.h>
#include <math.h>
#include <algorithm>
#include "../Toolbox/Util/Random.h"

/**
* @class Network
* 
* 	Base class to represent a network structure. Assumed that a network is made up of (weighted) edges and vertices. If edges
*		are conceptually unweighted, they should all be equally weighted (usually with a weight of 1). Derived classes handle 
* 		directedness of edges.
* 
* 	Networks are also responsible for tracking the community structure associated with them
*/
class Network{
public:
  Network(std::tr1::shared_ptr< Graph > g);
  /**
   * @fn ~Network()
   * Empty Destructor
   */
  ~Network(){};
  std::tr1::shared_ptr<Graph> getStructure(){return G;};
  void addCommunity(std::tr1::shared_ptr< Community > c){communities.insert(c);};
 
  void printCommunities(int minSize, std::ostream& fout);
  void loadCommunities(const string& filename, const string& delimiters);

  double minCommunity(){
    double minSize = G->nvertices();

    for (it_c = communities.begin(); it_c != communities.end(); it_c++){
      if ((*it_c)->size() < minSize ){
	minSize = (*it_c)->size();
      }
    }

    return minSize;
  };
  int numCommunities(){return communities.size();};

  /*Access methods for the list of communities and the participation calculation (used for evolution detection)*/
  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator cFirst(){return communities.begin();};
  std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >::iterator cLast(){return communities.end();};

  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator pFirst() { 
    return participation.begin();
  }
  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator pLast() { 
    return participation.end();
  }

  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator pMember(Graph::vertex v){
    
    std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator it_p;

    if ((it_p = participation.find(v)) == participation.end()){
      //cerr << "Vertex not found in participation" << endl;
      return it_p;
    }

    return it_p;
  }

  void getLongestEvolutions(set < EvolutionPtr >& evolutions, const int& minSize);
  void getLongestEvolutions(const int& minSize){
    getLongestEvolutions(E, minSize);
  };
  void getCurrentEvolutions(set < EvolutionPtr >& evolutions, const int& minSize);
  void ClearEvolutions(){E.clear();}
  void AddEvolution(EvolutionPtr ep){E.insert(ep);}

 
  /**Evolution Functions**/
  void calcParticipation();

  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator p_begin(){return participation.begin();}
  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator p_end(){return participation.end();}

  double participationSize(){return participation.size();};
  void calcSimilarity(std::tr1::shared_ptr< Network > other, const double& threshold);
  void printEvolutions(bool careAboutParents, int minLength);

private:
  std::tr1::shared_ptr<Graph> G;
  std::set < std::tr1::shared_ptr< Community >, Community::cmp_comm > communities;
  std::set < std::tr1::shared_ptr< Community >, Community::cmp_comm >::iterator it_c;

  std::set < EvolutionPtr > E; // That start in this network

  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string> participation;
  std::map < Graph::vertex, set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, cmp_tr1_string>::iterator it_p;
};

/**Functions to help with NMI calculations */
double h(double A, double N);
double H(double a, double b, double c, double d, double n);

double HXY ( std::tr1::shared_ptr< Network > A, std::tr1::shared_ptr < Network > B, double n );
double Hx ( std::tr1::shared_ptr < Network > A, double n );
double NMI ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B );


double Nodes ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B );
double BestMove ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B );
double FScore ( std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B, double threshold );

std::map < std::tr1::shared_ptr < Community >, std::set < std::tr1::shared_ptr < Community >, Community::cmp_comm >, Community::cmp_comm > CommunityMatch(std::tr1::shared_ptr < Network > A, std::tr1::shared_ptr < Network > B );

#endif
