#include "Network.h"
#include "Graph.h"
#include "../../Snap/snap-core/Snap.h"

/**
 * @class SNAPNetwork
 *
 * Helper class to transform the Network class into a structure that can
 *   be used with Stanford's library. Helpful for certain things like the
 *   centrality calculations
 */
class SNAPNetwork{

 public:

  SNAPNetwork(){  }

  /**
   * @fn SNAPNetwork(std::tr1::shared_ptr< Network > net)
   * 
   * Converts an initialized network to a SNAP network
   *
   * @param net Network structure to convert to Stanford's structure
   */
  SNAPNetwork(std::tr1::shared_ptr< Network > net){
    nextID = 0;
    
    network = TUNGraph::New(); //Initialize Stanford's structure
    
    //Add each network vertex while tracking the mapping from vertex to ID number
    for (Graph::vertex_iterator it_v = net->getStructure()->vBegin(); it_v != net->getStructure()->vEnd(); it_v++){
      IDMap.insert(pair< std::tr1::shared_ptr < const string >, int >(it_v->first, nextID));
      network->AddNode(nextID++);
    }
    
    //Add edges to the Stanford structure
    for (Graph::vertex_iterator it_v = net->getStructure()->vBegin(); it_v != net->getStructure()->vEnd(); it_v++){
      Graph::edge_iterator it_e;
      for (it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
	network->AddEdge(IDMap.find(it_v->first)->second, IDMap.find(it_e->first)->second);
      }
    }
    
  }


  /**
   * @fn SNAPNetwork(std::tr1::shared_ptr< Community > com, std::tr1::shared_ptr < Network > net)
   *
   * Create a Stanford structure network only considering the internal edges and vertices of the given community
   *
   * @param com Community to consider
   * @param net Network the community comes from
   */
  SNAPNetwork(std::tr1::shared_ptr< Community > com, std::tr1::shared_ptr < Network > net){
    nextID = 0;
    
    network = TUNGraph::New();
    
    for (Graph::vertex_iterator it_v = net->getStructure()->vBegin(); it_v != net->getStructure()->vEnd(); it_v++){
      if (com->hasMember(it_v->first)){
	IDMap.insert(pair< std::tr1::shared_ptr < const string >, int >(it_v->first, nextID));
	network->AddNode(nextID++);
      }
    }

    for (Graph::vertex_iterator it_v = net->getStructure()->vBegin(); it_v != net->getStructure()->vEnd(); it_v++){
      if (!(com->hasMember(it_v->first)))
	continue;

      Graph::edge_iterator it_e;
      for (it_e = it_v->second.begin(); it_e != it_v->second.end(); it_e++){
	if (com->hasMember(it_e->first))
	  network->AddEdge(IDMap.find(it_v->first)->second, IDMap.find(it_e->first)->second);
      }
    }
   
  }

  double Degree(std::tr1::shared_ptr< const string > n){
    if ((it_id = IDMap.find(n)) == IDMap.end()){
      cerr << "Could not find node " << *n << endl;
      exit(3);
    }

    return TSnap::GetDegreeCentr(network, it_id->second);
  }

  double Closeness(std::tr1::shared_ptr< const string > n){
    if ((it_id = IDMap.find(n)) == IDMap.end()){
      cerr << "Could not find node " << *n << endl;
      exit(3);
    }

    return TSnap::GetClosenessCentr(network, it_id->second);
  }

 private:
  std::map < std::tr1::shared_ptr < const string >, int, cmp_tr1_string > IDMap;
  std::map < std::tr1::shared_ptr < const string >, int, cmp_tr1_string >::iterator it_id;  

  int nextID;
  PUNGraph network;
};
