#include "TimeSeries.h"

/**
 * @fn void TimeSeries::LoadData(const std::string& filename, const std::string& delimiterA, const std::string& delimiterB, const double& wt)
 * @param filename File to load from. Each line should have a network file, followed by a space, and the community file associated with the network.
 * @param delimiterA Delimiter used in network files
 * @param delimiterB Delimiter in community files
 * @param wt Minimum weight for edges
 * 
 * Loads multiple Network and Community files
 */
void TimeSeries::LoadData(const std::string& filename, const std::string& delimiterA, const std::string& delimiterB, const double& wt, const bool positive)
{
  std::ifstream fin;
  
  if (!openFile(&fin, filename)){
    std::cerr << "Could not open file " << filename << std::endl;
    exit(19);
  }
  
  std::vector <string> fields;
  
  while (fline_tr(&fin, &fields, " ")){
    if (fields.size() != 2)
      continue;
    
    std::tr1::shared_ptr<Graph> structure(new UndirectedGraph());    
    structure->LoadStructure(fields[0], delimiterA, wt, positive);

    std::tr1::shared_ptr<Network> net(new Network(structure));

    net->loadCommunities(fields[1], delimiterB);
    
    timestep.push_back(net);
  }
}

/**
 * @fn void TimeSeries::LoadCommunityData(const std::string& filename, const std::string& delimiter)
 * @param filename File to load from. One community file per line, in chronological order.
 * @param delimiter Delimiter used in community files
 * 
 * Loads multiple community data files.
 
void TimeSeries::LoadCommunityData(const std::string& filename, const std::string& delimiter)
{
  std::ifstream fin;
  
  if (!openFile(&fin, filename)){
    std::cerr << "Could not open file " << filename << std::endl;
    exit(19);
  }
  
  std::vector <string> fields;
  
  while (fline_tr(&fin, &fields, delimiter)){
    if (fields.size() != 1)
      continue;
    
    std::tr1::shared_ptr<Network> net(new Network());
    net->loadCommunities(fields[0], delimiter);
    
    timestep.push_back(net);
    
    cerr << "Timestep " << timestep.size() << " Load Complete" << endl;
  }
}

/**
 * @fn void TimeSeries::BestMatch(const int& step, const int& CID)
 * @param step Timestep to start from
 * @param CID Community to start from
 * 
 * Starts from the given community in the given timestep and prints out the communities in the following 
 * 	timesteps that are closest to the starting community (w/r/t Jaccard Index)
 */
/*void TimeSeries::BestMatch(const int& step, const int& CID)
{
  std::tr1::shared_ptr<Community> c = timestep[step]->getCommunity(CID);
  c->print();
  std::cout << std::endl;
  
  for (int t = 0; t < timestep.size(); t++){
    cout << "Timestep " << t << endl;
    timestep[t]->bestMatch(c)->print();
    cout << endl;
  }
}

void TimeSeries::BestMatch(const int& step)
{
  for (std::set < std::tr1::shared_ptr<Community>, Community::cmp_comm >::const_iterator it_c = timestep[step]->firstCommunity();
       it_c != timestep[step]->lastCommunity();
      it_c++){    
    cout << "Target : " << endl;
    (*it_c)->print();
    cout << endl;
    
    for (int t = step; t < timestep.size(); t++){
      timestep[t]->bestMatch((*it_c))->print();
    }
    
    cout << "=====================================" << endl;
  }
}

/**
 * @fn void TimeSeries::CalculateSimilarities(const int& stepSize, const double& threshold, const int& minLength)
 * @param stepSize Difference between compared timesteps
 * @param threshold Minimum acceptable similarity (w/r/t Jaccard) for community pairs
 * @param minLength Minimum acceptable evolution length
 * 
 * Finds the valid evolutionary chains within the given restrictions. The calculation starts with all communities in the final time step being of length 1, 
 * 	and continues towards the first time step, prepending communities to evolution chains. The backwards nature is there to make tracking full evolution length
 * 	easier.
 */


void TimeSeries::CalculateSimilarities(const int& stepSize, const double& threshold)
{ 
  for (int i = 0; i < timestep.size(); i++){
    timestep[i]->calcParticipation();
  }
  
  for (int i = timestep.size() - stepSize - 1; i >= 0; i--){
    timestep[i]->calcSimilarity(timestep[i+stepSize], threshold);
  }

  for (int i = 0; i < timestep.size(); i++){
    timestep[i]->getLongestEvolutions(0);
  }
}

void TimeSeries::PrintEvolutions(const int& minLength){
  for (int i = 0; i < timestep.size(); i++){
    cout << "Timestep " << i << endl;
    timestep[i]->printEvolutions(true, minLength);
  }
}

double CosSim(map < Graph::vertex, int, cmp_tr1_string > A, map < Graph::vertex, int, cmp_tr1_string > B){
  double As = 0, Bs = 0, ABs = 0;
  
  map < Graph::vertex, int, cmp_tr1_string >::iterator it_a, it_b;
  
  for (it_a = A.begin(); it_a != A.end(); it_a++){
    As += it_a->second * it_a->second;
    if ( (it_b = B.find(it_a->first)) != B.end() ){
      ABs += it_a->second * it_b->second;
    }
  }

  for (it_b = B.begin(); it_b != B.end(); it_b++){
    Bs += it_b->second * it_b->second;
  }

  return ABs / (sqrt(As) + sqrt(Bs));
}

void TimeSeries::ExtendEvolutions(const int& minSize, const int& timegap, const int& stubSize, const double& threshold){
  map < int, set < EvolutionPtr > > starts, ends;
  map < int, set < EvolutionPtr > >::iterator it_m, it_n;
  
  set < EvolutionPtr >::iterator it_s, it_t;
  
  //Get all evolutions
  for (int i = 0; i < timestep.size(); i++){
    set < EvolutionPtr > currentSet;    
    timestep[i]->getCurrentEvolutions(currentSet, minSize);
   
    pair < map < int, set < EvolutionPtr > >::iterator, bool > res = starts.insert(pair < int, set < EvolutionPtr > > (i, currentSet));
    
    set < EvolutionPtr >::iterator it_s;
    for (it_s = currentSet.begin(); it_s != currentSet.end(); it_s++){
      int endpoint = i + (*it_s)->size() - 1;
      
      if ((it_m = ends.find(endpoint)) == ends.end()){
	set < EvolutionPtr > emptySet;
	emptySet.insert(*it_s);
	ends.insert(pair < int, set < EvolutionPtr > > (endpoint, emptySet));
      } else {
	it_m->second.insert(*it_s);
      }
    }
  }
  
  //Go through and merge potentials
  for ( int i = timestep.size(); i >= 0; i--)  {
    if ( (it_m = starts.find(i)) == starts.end() ){
      continue;
    }

    if ( (it_n = ends.find(i - timegap)) == ends.end() ){
      for (it_s = it_m->second.begin(); it_s != it_m->second.end(); it_s++){
	evolutions.insert(*it_s);
      }
      continue;
    }

    set < int > notCarried;
   
    for (it_s = it_n->second.begin(); it_s != it_n->second.end(); it_s++){
      (*it_s)->calcEndVector(stubSize);

      int best = 0;
      EvolutionPtr mergewith;
      int q = 0;
      int mergeID = 0;
      bool merged = false;
      
      for (it_t = it_m->second.begin(); it_t != it_m->second.end(); it_t++){
	(*it_t)->calcBeginVector(stubSize);
	
	/**
	 *If cosine sim between vectors > threshold, Merge
	 */
	if (CosSim((*it_s)->getPVector(), (*it_t)->getPVector()) >= threshold){
	  merged = true;
	  if ((*it_t)->size() > best){
	    best = (*it_t)->size();
	    mergewith = *it_t;
	    mergeID = q;
	  }
	}
	q++;
      }
      
      if (merged) {
	(*it_s)->merge(mergewith, timegap-1);
	notCarried.insert(mergeID);
      }
    }

    int d = 0;
    
    timestep[i]->ClearEvolutions();
    for (it_t = it_m->second.begin(); it_t != it_m->second.end(); it_t++){
      if (notCarried.find(d) == notCarried.end())
	timestep[i]->AddEvolution(*it_t);
      d++;
    }    
  }
}
