#include "Graph.h"

/**
 * @fn LoadNetwork(const std::string& filename, const std::string& delimiter, const double& weightThreshold)
 *
 * Loads network structure from an external text file. Text file should have the form where each line is:
 *     <Node1><delimiter><Node2><delimiter><Weight>
 * Both directions of the edge are stored in the structure. (Node1->Node2 is an edge and Node2->Node1 is an edge) 
 *
 * @param filename - Text file to load from
 * @param delimiter - Character(s) used to separate entries on a line
 * @param weightThreshold - Minimum allowed weight for an edge
 * @param positive - If true, will only consider edges with positive weights
 */
void UndirectedGraph::LoadStructure(const std::string& filename, const std::string& delimiter, const double& weightThreshold, const bool positive)
{
  /** Open the file to read from */
  std::ifstream fin;
  
  if (!openFile(&fin, filename)){
    cerr << "Could not open file " << filename << endl;
    exit(2);
  }
  
  std::vector < std::string > fields;
  
  /**Read through each line*/
  while (fline_tr(&fin, &fields, delimiter)){
    double weight = str_to<double>(fields[2]); // Extract weight

    //Positivity check (when applicable)
    if (positive && (weight <= 0))
      continue;

    //Check Weight Threhsold
    if (weight > 0 && weight < weightThreshold)
      continue;
    if (weight <= 0 && -weight < weightThreshold)
      continue;
    
    //Extract node information
    vertex currentBase(new std::string(fields[0]));
    vertex currentTarget(new std::string(fields[1]));
    
    vertex_iterator it_n, it_nn;
    edge_iterator it_e, it_ee;
    

    //Add Base->Target
    if ((it_n = G.find(currentBase)) != G.end()){
      if ((it_e = it_n->second.find(currentTarget)) != it_n->second.end()){
	if ((it_nn = G.find(currentTarget)) != G.end()){
	  if ( (it_ee = it_nn->second.find(currentBase)) != it_nn->second.end() ){
	    it_e->second = weight;
	  } else {
	    cerr << "Unbalanced network information : " << *currentBase << " - " << *currentTarget << " edge" << endl;
	  }
	} else {
	  cerr << "Unbalanced network information : " << *currentBase << " - " << *currentTarget << " edge" << endl;
	} 
	
      } else {
	it_n->second.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentTarget, weight));
      }
    } else {
      edgelist temp;
      temp.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentTarget, weight));
      G.insert(make_pair<std::tr1::shared_ptr<const std::string>, map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string > > (currentBase, temp));
    }
    
    //Add Target->Base
    if ((it_n = G.find(currentTarget)) != G.end()){
      if ((it_e = it_n->second.find(currentBase)) != it_n->second.end()){
	if ((it_nn = G.find(currentBase)) != G.end()){
	  if ( (it_ee = it_nn->second.find(currentTarget)) != it_nn->second.end() ){
	    it_e->second = weight;
	  } else {
	    cerr << "Unbalanced network information : " << *currentBase << " - " << *currentTarget << " edge" << endl;
	  }
	} else {
	  cerr << "Unbalanced network information : " << *currentBase << " - " << *currentTarget << " edge" << endl;
	}
      } else {
	it_n->second.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentBase, weight));
      }
    } else {
      edgelist temp;
      temp.insert(make_pair<std::tr1::shared_ptr<const std::string>, double>(currentBase, weight));
      G.insert(make_pair<std::tr1::shared_ptr<const std::string>, map < std::tr1::shared_ptr<const std::string>, double, cmp_tr1_string > >(currentTarget, temp));
    }
  }
}

/**
 * void UndirectedGraph::PrintStructure(ostream& fout)void UndirectedGraph::PrintStructure(ostream& fout)
 *
 * Prints the structure to a text file. Each line is <Node>,<Node>,<Weight>
 *
 * @param fout Where to print the file
 */
void UndirectedGraph::PrintStructure(ostream& fout){
  vertex_iterator it_n;
  edge_iterator it_e;
  
  cmp_tr1_string cp;
    
  for (it_n = G.begin(); it_n != G.end(); it_n++){
    for (it_e = it_n->second.begin(); it_e != it_n->second.end(); it_e++){
      if (cp(it_n->first, it_e->first))
	fout << *(it_n->first) << ", " << *(it_e->first) << ", " << it_e->second << endl;
    }
  }
}

/**
 * unsigned int UndirectedGraph::nedges()
 *
 * Count the number of edges in the network
 */
unsigned int UndirectedGraph::nedges(){
  return (Graph::nedges() / 2);
}
