#ifndef TIMER_JT
#define TIMER_JT

#include <sys/time.h>
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

/**
 * @class Timer
 * 
 * To help time pieces of code in a more organized and readable manner
 */
class Timer{
	public:
	  /**
	   * @fn Timer(string filename)
	   * 
	   * @param filename File to print out the times to
	   * 
	   * Basic initialization Constructor
	   */
 Timer(string filename):initialized(false), completed(false){
    tout.open(filename.c_str());
  }
	
	/**
	   * @fn ~Timer()
	   * 
	   * Basic Destructor
	   */
		virtual ~Timer(){
			tout.close();
		}
		
		/**
		 * @fn void Begin()
		 * 
		 * Gets current time as the 'start' or 'base' time
		 */
		void Begin(){
			gettimeofday(&init, NULL);
			initialized = true;
			completed = false;
		}
		
		/**
		 * @fn void End()
		 * 
		 * Gets current time as the 'finish'
		 */
		void End(){
			gettimeofday(&term, NULL);
			completed = true;
			dif = diff(init, term);
		}  
		
		/**
		 * @fn void PrintTime(string prompt = "Time: ")
		 * @param prompt What to prompt the output with
		 * 
		 *  Calculates the difference between the start and finish times, and prints results to the output file
		 */		
		void PrintTime(string prompt = "Time: "){
			if (initialized && completed){
				tout << prompt << dif.tv_sec << ":" << dif.tv_usec <<  " seconds." << endl;
			} else {
				tout << "Bad Timing" << endl;
			}
			tout.flush();
		}
	
	protected:
		bool initialized, completed;
		struct timeval init, term, dif;
		ofstream tout;
		
		/**
		 * @fn struct timeval diff(struct timeval start, struct timeval end)
		 * @param start Start time
		 * @param end Ending time
		 * 
		 * @return Difference between the two given times (end - start)
		 */
		struct timeval diff(struct timeval start, struct timeval end)
		{
		  unsigned long total = ((end.tv_sec * 1000000) - (start.tv_sec * 1000000)) + end.tv_usec - start.tv_usec;

		  struct timeval difference;
		  difference.tv_sec = total / 1000000;
		  difference.tv_usec = total % 1000000;

		  return difference;
		}

		
};

/**
 * @class SplitTimer
 * 
 * For use when timing is done in chunks, not continuous code.
 */
class SplitTimer: public Timer{
	public:
	  /**
	   * @fn SplitTimer(string filename)
	   * @param filename File to print output to
	   * 
	   * Basic initialization constructor
	   */
		SplitTimer(string filename):Timer(filename){
			allofit.tv_sec = 0; 
			allofit.tv_usec = 0;
		}
	
	/**
	 * @fn void Begin()
	 * 
	 * Starts the timer
	 */
		void Begin(){
			((Timer*)this)->Begin();
		}
	
	/**
	 * @fn void End()
	 * 
	 * Ends timer and add the difference from the start to the total time keeper
	 */
		void End(){
			((Timer*)this)->End();
			dif = diff(init, term);
			long unsigned int total = (dif.tv_sec * 1000000) + dif.tv_usec + (allofit.tv_sec * 1000000) + allofit.tv_usec;
			allofit.tv_sec = total / 1000000;
			allofit.tv_usec = total % 1000000;
		}

		void PrintTime(string prompt = "Time : "){
		  dif = allofit;
		  static_cast<Timer*>(this)->PrintTime(prompt);
		}
	       
		
	private:
		struct timeval allofit;
};

//Need an interval ('lap') timer

class LapTimer: public Timer{
	public:
	  /**
	   * @fn LapTimer(string filename)
	   * @param filename File to print output to
	   * 
	   * Basic initialization constructor
	   */
 LapTimer(string filename):Timer(filename){}
	
	/**
	 * @fn void Begin()
	 * 
	 * Starts the timer
	 */
		void Begin(){
			((Timer*)this)->Begin();
		}
	
	/**
	 * @fn void End()
	 * 
	 * Ends timer and posts the run time to cerr
	 */
		void End(string prompt = "Lap Time : "){
			((Timer*)this)->End();
			dif = diff(init, term);
			if (initialized && completed)
			  cerr << prompt << dif.tv_sec << ":" << dif.tv_usec << " seconds." << endl;
			else
			  cerr << "Bad Timing" << endl;
		}
};


#endif
